# Copyright (C) 2001-2006 Python Software Foundation
# Author: Barry Warsaw
# Contact: email-sig@python.org

"""Encodings and related functions."""

__all__ = [
    "encode_7or8bit",
    "encode_base64",
    "encode_noop",
    "encode_quopri",
]


from base64 import encodebytes as _bencode
from quopri import encodestring as _encodestring


def _qencode(s):
    enc = _encodestring(s, quotetabs=True)
    # Must encode spaces, which quopri.encodestring() doesn't do
    return enc.replace(b" ", b"=20")


def encode_base64(msg):
    """Encode the message's payload in Base64.

    Also, add an appropriate Content-Transfer-Encoding header.
    """
    orig = msg.get_payload(decode=True)
    encdata = str(_bencode(orig), "ascii")
    msg.set_payload(encdata)
    msg["Content-Transfer-Encoding"] = "base64"


def encode_quopri(msg):
    """Encode the message's payload in quoted-printable.

    Also, add an appropriate Content-Transfer-Encoding header.
    """
    orig = msg.get_payload(decode=True)
    encdata = _qencode(orig)
    msg.set_payload(encdata)
    msg["Content-Transfer-Encoding"] = "quoted-printable"


def encode_7or8bit(msg):
    """Set the Content-Transfer-Encoding header to 7bit or 8bit."""
    orig = msg.get_payload(decode=True)
    if orig is None:
        # There's no payload.  For backwards compatibility we use 7bit
        msg["Content-Transfer-Encoding"] = "7bit"
        return
    # We play a trick to make this go fast.  If encoding/decode to ASCII
    # succeeds, we know the data must be 7bit, otherwise treat it as 8bit.
    try:
        if isinstance(orig, str):
            orig.encode("ascii")
        else:
            orig.decode("ascii")
    except UnicodeError:
        charset = msg.get_charset()
        output_cset = charset and charset.output_charset
        # iso-2022-* is non-ASCII but encodes to a 7-bit representation
        if output_cset and output_cset.lower().startswith("iso-2022-"):
            msg["Content-Transfer-Encoding"] = "7bit"
        else:
            msg["Content-Transfer-Encoding"] = "8bit"
    else:
        msg["Content-Transfer-Encoding"] = "7bit"


def encode_noop(msg):
    """Do nothing."""



# Very simple HTTP client example:
# - Connects to a server.
# - Sends a HTTP request.
# - Reads the result.
#
# This example works in both MicroPython and CPython.
#
# To implement an HTTP client using less code, use mip to install the requests package:
# https://github.com/micropython/micropython-lib/tree/master/python-ecosys/requests

import socket


# `addr_family` selects IPv4 vs IPv6: 0 means either, or use
# socket.AF_INET or socket.AF_INET6 to select a particular one.
def main(url, addr_family=0, use_stream=False):
    # Split the given URL into components.
    proto, _, host, path = url.split(b"/", 3)
    assert proto == b"http:"

    # Lookup the server address, for the given family and socket type.
    ai = socket.getaddrinfo(host, 80, addr_family, socket.SOCK_STREAM)
    print("Address infos:", ai)

    # Select the first address.
    ai = ai[0]

    # Create a socket with the server's family, type and proto.
    s = socket.socket(ai[0], ai[1], ai[2])

    # Connect to the server.
    addr = ai[-1]
    print("Connect address:", addr)
    s.connect(addr)

    # Send request and read response.
    request = b"GET /%s HTTP/1.0\r\nHost: %s\r\n\r\n" % (path, host)
    if use_stream:
        # MicroPython socket objects support stream (aka file) interface
        # directly, but the line below is needed for CPython.
        s = s.makefile("rwb", 0)
        s.write(request)
        print(s.read())
    else:
        s.send(request)
        print(s.recv(4096))

    # Close the socket.
    s.close()


main(b"http://www.google.com/")



#!/usr/bin/env python3
#
# This file is part of the MicroPython project, http://micropython.org/
#
# The MIT License (MIT)
#
# Copyright (c) 2020 Damien P. George
# Copyright (c) 2023 Jim Mussared
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# This is just a wrapper around running ruff format, so that code formatting can be
# invoked in the same way as in the main repo.

import os
import subprocess

# Path to repo top-level dir.
TOP = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))


def main():
    command = ["ruff", "format", "."]
    subprocess.check_call(command, cwd=TOP)


if __name__ == "__main__":
    main()



class LED:
    def __init__(self, id):
        self.f = open("/sys/class/leds/%s/brightness" % id, "r+b")

    def on(self):
        self.f.write(b"255")

    def off(self):
        self.f.write(b"0")

    def get(self):
        self.f.seek(0)
        return int(self.f.read())

    def toggle(self):
        v = self.get()
        if v:
            self.off()
        else:
            self.on()



#!/usr/bin/env python

import os
import re
import sys

sys.path.insert(0, os.path.join(os.path.dirname(__file__), "../../../tools"))
import boardgen


class RenesasRaPin(boardgen.Pin):
    def __init__(self, cpu_pin_name):
        super().__init__(cpu_pin_name)

        self._port = int(cpu_pin_name[1], 16)
        self._bit = int(cpu_pin_name[2:])

        self._adc_name = None
        self._adc_bits = 0
        self._adc_channel = 0

    def add_adc(self, adc):
        # The previous version of make-pins.py handled len==4 to mean
        # adc_bits=8, but this is not used in any of the current af.csv files,
        # so this needs to be validated.
        assert len(adc) == 5 and (adc.startswith("AN0") or adc.startswith("AN1"))
        self._adc_name = adc
        self._adc_bit = 12
        self._adc_channel = int(adc[2:])

    # Called for each AF defined in the csv file for this pin.
    def add_af(self, af_idx, af_name, af):
        if af_idx == 0 and af:
            assert af_name == "Analog"
            self.add_adc(af)

    # Use the PIN() macro defined in ra_pin_prefix.c for defining the pin
    # objects.
    def definition(self):
        # PIN(p_name, p_pin, p_adc)
        adc = "&pin_{}_adc_obj".format(self.name()) if self._adc_name else "NULL"
        return "PIN({:s}, {:d}, {:s})".format(self.name(), self._port * 16 + self._bit, adc)

    # This will be called at the start of the output (after the prefix). Use
    # it to emit the ADC objects (via the PIN_ADC() macro).
    def print_source(self, out_source):
        # PIN_ADC(p_name, p_pin, adc_bit, adc_channel)
        if self._adc_name:
            print(
                "const machine_pin_adc_obj_t pin_{}_adc_obj = PIN_ADC({:s}, {:d}, {:d}, {:d});".format(
                    self.name(),
                    self._adc_name,
                    self._port * 16 + self._bit,
                    self._adc_bits,
                    self._adc_channel,
                ),
                file=out_source,
            )

    # RA cpu names must be "PXNN", where X is hexadecimal port, NN is decimal bit offset.
    @staticmethod
    def validate_cpu_pin_name(cpu_pin_name):
        boardgen.Pin.validate_cpu_pin_name(cpu_pin_name)

        if not re.match("^P[0-9A-F][0-9][0-9]$", cpu_pin_name):
            raise boardgen.PinGeneratorError("Invalid cpu pin name '{}'".format(cpu_pin_name))


class RenesasRaPinGenerator(boardgen.PinGenerator):
    def __init__(self):
        # Use custom pin type above, and also enable the --af-csv argument so
        # that add_af gets called on each pin.
        super().__init__(
            pin_type=RenesasRaPin,
            enable_af=True,
        )

    # Override the default implementation just to change the default arguments.
    def parse_af_csv(self, filename):
        return super().parse_af_csv(filename, header_rows=1, pin_col=0, af_col=3)

    # Renesas-specific behavior, we use pin_PNNN for the cpu names, but
    # pyb_pin_FOO for the board names.
    def board_name_define_prefix(self):
        return "pyb_"


if __name__ == "__main__":
    RenesasRaPinGenerator().main()



# test memoryview slicing beyond the limit of what memoryview can internally index

try:
    from sys import maxsize
    from uctypes import bytearray_at

    memoryview
except:
    print("SKIP")
    raise SystemExit

if maxsize <= 0xFFFF_FFFF:
    slice_max = 0xFF_FFFF
else:
    slice_max = 0xFF_FFFF_FFFF_FFFF

buf = bytearray_at(0, slice_max + 2)
mv = memoryview(buf)

# this should work
print(mv[slice_max : slice_max + 1])

# this should overflow the internal index for memoryview slicing
try:
    print(mv[slice_max + 1 : slice_max + 2])
except OverflowError:
    print("OverflowError")



# test builtin sorted
try:
    sorted
    set
except:
    print("SKIP")
    raise SystemExit

print(sorted(set(range(100))))
print(sorted(set(range(100)), key=lambda x: x + 100*(x % 2)))

# need to use keyword argument
try:
    sorted([], None)
except TypeError:
    print("TypeError")



import vfs
from flashbdev import bdev


def check_bootsec():
    buf = bytearray(bdev.ioctl(5, 0))  # 5 is SEC_SIZE
    bdev.readblocks(0, buf)
    empty = True
    for b in buf:
        if b != 0xFF:
            empty = False
            break
    if empty:
        return True
    fs_corrupted()


def fs_corrupted():
    import time
    import micropython

    # Allow this loop to be stopped via Ctrl-C.
    micropython.kbd_intr(3)

    while 1:
        print(
            """\
The filesystem appears to be corrupted. If you had important data there, you
may want to make a flash snapshot to try to recover it. Otherwise, perform
factory reprogramming of MicroPython firmware (completely erase flash, followed
by firmware programming).
"""
        )
        time.sleep(3)


def setup():
    check_bootsec()
    print("Performing initial setup")
    if bdev.info()[4] == "vfs":
        vfs.VfsLfs2.mkfs(bdev)
        fs = vfs.VfsLfs2(bdev)
    elif bdev.info()[4] == "ffat":
        vfs.VfsFat.mkfs(bdev)
        fs = vfs.VfsFat(bdev)
    vfs.mount(fs, "/")
    with open("boot.py", "w") as f:
        f.write(
            """\
# This file is executed on every boot (including wake-boot from deepsleep)
#import esp
#esp.osdebug(None)
#import webrepl
#webrepl.start()
"""
        )
    return fs



#
# MicroPython http_server_simplistic.py example
#
# This example shows how to write the smallest possible HTTP
# server in MicroPython. With comments and convenience code
# removed, this example can be compressed literally to ten
# lines. There's a catch though - read comments below for
# details, and use this code only for quick hacks, preferring
# http_server.py for "real thing".
#
import socket


CONTENT = b"""\
HTTP/1.0 200 OK

Hello #%d from MicroPython!
"""


def main():
    s = socket.socket()

    # Bind to (allow to be connected on ) all interfaces. This means
    # this server will be accessible to other hosts on your local
    # network, and if your server has direct (non-firewalled) connection
    # to the Internet, then to anyone on the Internet. We bind to all
    # interfaces to let this example work easily on embedded MicroPython
    # targets, which you will likely access from another machine on your
    # local network. Take care when running this on an Internet-connected
    # machine though! Replace "0.0.0.0" with "127.0.0.1" if in doubt, to
    # make the server accessible only on the machine it runs on.
    ai = socket.getaddrinfo("0.0.0.0", 8080)
    print("Bind address info:", ai)
    addr = ai[0][-1]

    # A port on which a socket listened remains inactive during some time.
    # This means that if you run this sample, terminate it, and run again
    # you will likely get an error. To avoid this timeout, set SO_REUSEADDR
    # socket option.
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

    s.bind(addr)
    s.listen(5)
    print("Listening, connect your browser to http://<this_host>:8080/")

    counter = 0
    while True:
        res = s.accept()
        client_s = res[0]
        client_addr = res[1]
        print("Client address:", client_addr)
        print("Client socket:", client_s)
        # We assume here that .recv() call will read entire HTTP request
        # from client. This is usually true, at least on "big OS" systems
        # like Linux/MacOS/Windows. But that doesn't have to be true in
        # all cases, in particular on embedded systems, when there can
        # easily be "short recv", where it returns much less than requested
        # data size. That's why this example is called "simplistic" - it
        # shows that writing a web server in Python that *usually works* is
        # ten lines of code, and you can use this technique for quick hacks
        # and experimentation. But don't do it like that in production
        # applications - instead, parse HTTP request properly, as shown
        # by http_server.py example.
        req = client_s.recv(4096)
        print("Request:")
        print(req)
        client_s.send(CONTENT % counter)
        client_s.close()
        counter += 1
        print()


main()



try:
    import random
except ImportError:
    print("SKIP")
    raise SystemExit

# check getrandbits returns a value within the bit range
for b in (1, 2, 3, 4, 16, 32):
    for i in range(50):
        assert random.getrandbits(b) < (1 << b)

# check that seed(0) gives a non-zero value
random.seed(0)
print(random.getrandbits(16) != 0)

# check that PRNG is repeatable
random.seed(1)
r = random.getrandbits(16)
random.seed(1)
print(random.getrandbits(16) == r)

# check that zero bits works
print(random.getrandbits(0))

# check that it throws an error for negative bits
try:
    random.getrandbits(-1)
except ValueError:
    print("ValueError")



# test passing a user-defined mapping as the argument to **

def foo(**kw):
    print(sorted(kw.items()))

class Mapping:
    def keys(self):
        # the long string checks the case of string interning
        return ['a', 'b', 'c', 'abcdefghijklmnopqrst']

    def __getitem__(self, key):
        if key == 'a':
            return 1
        else:
            return 2

foo(**Mapping())



from micropython import const
import io
import sys
import time

CRITICAL = const(50)
ERROR = const(40)
WARNING = const(30)
INFO = const(20)
DEBUG = const(10)
NOTSET = const(0)

_DEFAULT_LEVEL = const(WARNING)

_level_dict = {
    CRITICAL: "CRITICAL",
    ERROR: "ERROR",
    WARNING: "WARNING",
    INFO: "INFO",
    DEBUG: "DEBUG",
    NOTSET: "NOTSET",
}

_loggers = {}
_stream = sys.stderr
_default_fmt = "%(levelname)s:%(name)s:%(message)s"
_default_datefmt = "%Y-%m-%d %H:%M:%S"


class LogRecord:
    def set(self, name, level, message):
        self.name = name
        self.levelno = level
        self.levelname = _level_dict[level]
        self.message = message
        self.ct = time.time()
        self.msecs = int((self.ct - int(self.ct)) * 1000)
        self.asctime = None


class Handler:
    def __init__(self, level=NOTSET):
        self.level = level
        self.formatter = None

    def close(self):
        pass

    def setLevel(self, level):
        self.level = level

    def setFormatter(self, formatter):
        self.formatter = formatter

    def format(self, record):
        return self.formatter.format(record)


class StreamHandler(Handler):
    def __init__(self, stream=None):
        super().__init__()
        self.stream = _stream if stream is None else stream
        self.terminator = "\n"

    def close(self):
        if hasattr(self.stream, "flush"):
            self.stream.flush()

    def emit(self, record):
        if record.levelno >= self.level:
            self.stream.write(self.format(record) + self.terminator)


class FileHandler(StreamHandler):
    def __init__(self, filename, mode="a", encoding="UTF-8"):
        super().__init__(stream=open(filename, mode=mode, encoding=encoding))

    def close(self):
        super().close()
        self.stream.close()


class Formatter:
    def __init__(self, fmt=None, datefmt=None):
        self.fmt = _default_fmt if fmt is None else fmt
        self.datefmt = _default_datefmt if datefmt is None else datefmt

    def usesTime(self):
        return "asctime" in self.fmt

    def formatTime(self, datefmt, record):
        if hasattr(time, "strftime"):
            return time.strftime(datefmt, time.localtime(record.ct))
        return None

    def format(self, record):
        if self.usesTime():
            record.asctime = self.formatTime(self.datefmt, record)
        return self.fmt % {
            "name": record.name,
            "message": record.message,
            "msecs": record.msecs,
            "asctime": record.asctime,
            "levelname": record.levelname,
        }


class Logger:
    def __init__(self, name, level=NOTSET):
        self.name = name
        self.level = level
        self.handlers = []
        self.record = LogRecord()

    def setLevel(self, level):
        self.level = level

    def isEnabledFor(self, level):
        return level >= self.getEffectiveLevel()

    def getEffectiveLevel(self):
        return self.level or getLogger().level or _DEFAULT_LEVEL

    def log(self, level, msg, *args):
        if self.isEnabledFor(level):
            if args:
                if isinstance(args[0], dict):
                    args = args[0]
                msg = msg % args
            self.record.set(self.name, level, msg)
            handlers = self.handlers
            if not handlers:
                handlers = getLogger().handlers
            for h in handlers:
                h.emit(self.record)

    def debug(self, msg, *args):
        self.log(DEBUG, msg, *args)

    def info(self, msg, *args):
        self.log(INFO, msg, *args)

    def warning(self, msg, *args):
        self.log(WARNING, msg, *args)

    def error(self, msg, *args):
        self.log(ERROR, msg, *args)

    def critical(self, msg, *args):
        self.log(CRITICAL, msg, *args)

    def exception(self, msg, *args, exc_info=True):
        self.log(ERROR, msg, *args)
        tb = None
        if isinstance(exc_info, BaseException):
            tb = exc_info
        elif hasattr(sys, "exc_info"):
            tb = sys.exc_info()[1]
        if tb:
            buf = io.StringIO()
            sys.print_exception(tb, buf)
            self.log(ERROR, buf.getvalue())

    def addHandler(self, handler):
        self.handlers.append(handler)

    def hasHandlers(self):
        return len(self.handlers) > 0


def getLogger(name=None):
    if name is None:
        name = "root"
    if name not in _loggers:
        _loggers[name] = Logger(name)
        if name == "root":
            basicConfig()
    return _loggers[name]


def log(level, msg, *args):
    getLogger().log(level, msg, *args)


def debug(msg, *args):
    getLogger().debug(msg, *args)


def info(msg, *args):
    getLogger().info(msg, *args)


def warning(msg, *args):
    getLogger().warning(msg, *args)


def error(msg, *args):
    getLogger().error(msg, *args)


def critical(msg, *args):
    getLogger().critical(msg, *args)


def exception(msg, *args):
    getLogger().exception(msg, *args)


def shutdown():
    for k, logger in _loggers.items():
        for h in logger.handlers:
            h.close()
        _loggers.pop(logger, None)


def addLevelName(level, name):
    _level_dict[level] = name


def basicConfig(
    filename=None,
    filemode="a",
    format=None,
    datefmt=None,
    level=WARNING,
    stream=None,
    encoding="UTF-8",
    force=False,
):
    if "root" not in _loggers:
        _loggers["root"] = Logger("root")

    logger = _loggers["root"]

    if force or not logger.handlers:
        for h in logger.handlers:
            h.close()
        logger.handlers = []

        if filename is None:
            handler = StreamHandler(stream)
        else:
            handler = FileHandler(filename, filemode, encoding)

        handler.setLevel(level)
        handler.setFormatter(Formatter(format, datefmt))

        logger.setLevel(level)
        logger.addHandler(handler)


if hasattr(sys, "atexit"):
    sys.atexit(shutdown)



# ProS3 MicroPython Helper Library
# MIT license; Copyright (c) 2022 Seon Rozenblum - Unexpected Maker
#
# Project home:
# http://pros3.io

# Import required libraries
from micropython import const
from machine import Pin, ADC
import time

# ProS3 Hardware Pin Assignments

# Sense Pins
VBUS_SENSE = const(33)
VBAT_SENSE = const(10)

# RGB LED & LDO2 Pins
RGB_DATA = const(18)
LDO2 = const(17)

# SPI
SPI_MOSI = const(35)
SPI_MISO = const(37)
SPI_CLK = const(36)

# I2C
I2C_SDA = const(8)
I2C_SCL = const(9)


# Helper functions
def set_ldo2_power(state):
    """Enable or Disable power to the second LDO"""
    Pin(LDO2, Pin.OUT).value(state)


def get_battery_voltage():
    """
    Returns the current battery voltage. If no battery is connected, returns 4.2V which is the charge voltage
    This is an approximation only, but useful to detect if the charge state of the battery is getting low.
    """
    adc = ADC(Pin(VBAT_SENSE))  # Assign the ADC pin to read
    adc.atten(ADC.ATTN_2_5DB)  # Needs 2.5DB attenuation for max voltage of 1.116V w/batt of 4.2V
    measuredvbat = adc.read()
    measuredvbat /= 3657  # Divide by 3657 as we are using 2.5dB attenuation, which is max input of 1.25V = 4095 counts
    measuredvbat *= 4.2  # Multiply by 4.2V, our max charge voltage for a 1S LiPo
    return round(measuredvbat, 2)


def get_vbus_present():
    """Detect if VBUS (5V) power source is present"""
    return Pin(VBUS_SENSE, Pin.IN).value() == 1


# NeoPixel rainbow colour wheel
def rgb_color_wheel(wheel_pos):
    """Color wheel to allow for cycling through the rainbow of RGB colors."""
    wheel_pos = wheel_pos % 255

    if wheel_pos < 85:
        return 255 - wheel_pos * 3, 0, wheel_pos * 3
    elif wheel_pos < 170:
        wheel_pos -= 85
        return 0, wheel_pos * 3, 255 - wheel_pos * 3
    else:
        wheel_pos -= 170
        return wheel_pos * 3, 255 - wheel_pos * 3, 0



# Use built-in algorithms preferentially (on many ports this is just sha256).
try:
    from uhashlib import *
except ImportError:
    pass


# Add missing algorithms based on installed extensions.
def _init():
    for algo in ("sha224", "sha256", "sha384", "sha512"):
        if algo not in globals():
            try:
                # from ._{algo} import {algo}
                c = __import__("_" + algo, None, None, (), 1)
                globals()[algo] = getattr(c, algo)
            except ImportError:
                pass


_init()
del _init


def new(algo, data=b""):
    try:
        c = globals()[algo]
        return c(data)
    except KeyError:
        raise ValueError(algo)



"""
The MIT License (MIT)

Copyright (c) 2016-2019 shaoziyang <shaoziyang@micropython.org.cn>
Copyright (c) 2022 Ibrahim Abdelkader <iabdalkader@openmv.io>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

LPS22HB/HH pressure sensor driver for MicroPython.

Example usage:

import time
from lps22h import LPS22H
from machine import Pin, I2C

bus = I2C(1, scl=Pin(15), sda=Pin(14))
lps = LPS22H(bus, oneshot=False)

while (True):
    print("Pressure: %.2f hPa Temperature: %.2f C"%(lps.pressure(), lps.temperature()))
    time.sleep_ms(10)
"""
import machine
from micropython import const

_LPS22_CTRL_REG1 = const(0x10)
_LPS22_CTRL_REG2 = const(0x11)
_LPS22_STATUS = const(0x27)
_LPS22_TEMP_OUT_L = const(0x2B)
_LPS22_PRESS_OUT_XL = const(0x28)
_LPS22_PRESS_OUT_L = const(0x29)


class LPS22H:
    def __init__(self, i2c, address=0x5C, oneshot=False):
        self.i2c = i2c
        self.addr = address
        self.oneshot = oneshot
        self.buf = bytearray(1)
        # ODR=1 EN_LPFP=1 BDU=1
        self._write_reg(_LPS22_CTRL_REG1, 0x1A)
        self.set_oneshot_mode(self.oneshot)

    def _int16(self, d):
        return d if d < 0x8000 else d - 0x10000

    def _write_reg(self, reg, dat):
        self.buf[0] = dat
        self.i2c.writeto_mem(self.addr, reg, self.buf)

    def _read_reg(self, reg, width=8):
        self.i2c.readfrom_mem_into(self.addr, reg, self.buf)
        val = self.buf[0]
        if width == 16:
            val |= self._read_reg(reg + 1) << 8
        return val

    def _tigger_oneshot(self, b):
        if self.oneshot:
            self._write_reg(_LPS22_CTRL_REG2, self._read_reg(_LPS22_CTRL_REG2) | 0x01)
            self._read_reg(0x28 + b * 2)
            while True:
                if self._read_reg(_LPS22_STATUS) & b:
                    return
                machine.idle()

    def set_oneshot_mode(self, oneshot):
        self._read_reg(_LPS22_CTRL_REG1)
        self.oneshot = oneshot
        if oneshot:
            self.buf[0] &= 0x0F
        else:
            self.buf[0] |= 0x10
        self._write_reg(_LPS22_CTRL_REG1, self.buf[0])

    def pressure(self):
        if self.oneshot:
            self._tigger_oneshot(1)
        return (
            self._read_reg(_LPS22_PRESS_OUT_XL) + self._read_reg(_LPS22_PRESS_OUT_L, 16) * 256
        ) / 4096

    def temperature(self):
        if self.oneshot:
            self._tigger_oneshot(2)
        return self._int16(self._read_reg(_LPS22_TEMP_OUT_L, 16)) / 100

    def altitude(self):
        return (
            (((1013.25 / self.pressure()) ** (1 / 5.257)) - 1.0)
            * (self.temperature() + 273.15)
            / 0.0065
        )



# Test a task cancelling itself (currently unsupported)

try:
    import asyncio
except ImportError:
    print("SKIP")
    raise SystemExit


async def task():
    print("task start")
    global_task.cancel()


async def main():
    global global_task
    global_task = asyncio.create_task(task())
    try:
        await global_task
    except asyncio.CancelledError:
        print("main cancel")
    print("main done")


try:
    asyncio.run(main())
except RuntimeError as er:
    print(er)



# test async def

def dec(f):
    print('decorator')
    return f

# test definition with a decorator
@dec
async def foo():
    print('foo')

coro = foo()
try:
    coro.send(None)
except StopIteration:
    print('StopIteration')



# Test deflate module with stream errors.

try:
    # Check if deflate & IOBase are available.
    import deflate, io

    io.IOBase
except (ImportError, AttributeError):
    print("SKIP")
    raise SystemExit

# Check if compression is enabled.
if not hasattr(deflate.DeflateIO, "write"):
    print("SKIP")
    raise SystemExit

formats = (deflate.RAW, deflate.ZLIB, deflate.GZIP)

# Test error on read when decompressing.


class Stream(io.IOBase):
    def readinto(self, buf):
        print("Stream.readinto", len(buf))
        return -1


try:
    deflate.DeflateIO(Stream()).read()
except OSError as er:
    print(repr(er))

# Test error on write when compressing.


class Stream(io.IOBase):
    def write(self, buf):
        print("Stream.write", buf)
        return -1


for format in formats:
    try:
        deflate.DeflateIO(Stream(), format).write("a")
    except OSError as er:
        print(repr(er))

# Test write after close.


class Stream(io.IOBase):
    def write(self, buf):
        print("Stream.write", buf)
        return -1

    def ioctl(self, cmd, arg):
        print("Stream.ioctl", cmd, arg)
        return 0


try:
    d = deflate.DeflateIO(Stream(), deflate.RAW, 0, True)
    d.close()
    d.write("a")
except OSError as er:
    print(repr(er))

# Test error on write when closing.


class Stream(io.IOBase):
    def __init__(self):
        self.num_writes = 0

    def write(self, buf):
        print("Stream.write", buf)
        if self.num_writes >= 4:
            return -1
        self.num_writes += 1
        return len(buf)


for format in formats:
    d = deflate.DeflateIO(Stream(), format)
    d.write("a")
    try:
        d.close()
    except OSError as er:
        print(repr(er))



"""
categories: Core,Functions
description: User-defined attributes for functions are not supported
cause: MicroPython is highly optimized for memory usage.
workaround: Use external dictionary, e.g. ``FUNC_X[f] = 0``.
"""


def f():
    pass


f.x = 0
print(f.x)



# Basic test of tls.SSLContext get_ciphers() and set_ciphers() methods.

try:
    import tls

    tls.SSLContext.get_ciphers
except (ImportError, AttributeError):
    print("SKIP")
    raise SystemExit


ctx = tls.SSLContext(tls.PROTOCOL_TLS_CLIENT)

ciphers = ctx.get_ciphers()

for ci in ciphers:
    # Only print those ciphers know to exist on all ports.
    if ("TLS-ECDHE-ECDSA-WITH-AES" in ci or "TLS-RSA-WITH-AES" in ci) and "CBC" in ci:
        print(ci)

ctx.set_ciphers(ciphers[:1])

# Test error cases.

try:
    ctx.set_ciphers(ciphers[0])
except TypeError as e:
    print(e)

try:
    ctx.set_ciphers(["BAR"])
except OSError as e:
    print(e)



# Test creating an SSL connection and getting the peer certificate.

try:
    import io
    import os
    import socket
    import ssl
except ImportError:
    print("SKIP")
    raise SystemExit

PORT = 8000

# These are test certificates. See tests/README.md for details.
cert = cafile = "ec_cert.der"
key = "ec_key.der"

try:
    os.stat(cafile)
    os.stat(key)
except OSError:
    print("SKIP")
    raise SystemExit


# Server
def instance0():
    multitest.globals(IP=multitest.get_network_ip())
    s = socket.socket()
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.bind(socket.getaddrinfo("0.0.0.0", PORT)[0][-1])
    s.listen(1)
    multitest.next()
    s2, _ = s.accept()
    server_ctx = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
    server_ctx.load_cert_chain(cert, key)
    s2 = server_ctx.wrap_socket(s2, server_side=True)
    print(s2.read(16))
    s2.write(b"server to client")
    s2.close()
    s.close()


# Client
def instance1():
    s_test = ssl.wrap_socket(io.BytesIO(), server_side=True, do_handshake=False)
    s_test.close()
    if not hasattr(s_test, "getpeercert"):
        print("SKIP")
        raise SystemExit

    multitest.next()
    s = socket.socket()
    s.connect(socket.getaddrinfo(IP, PORT)[0][-1])
    client_ctx = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
    client_ctx.verify_mode = ssl.CERT_REQUIRED
    client_ctx.load_verify_locations(cafile=cafile)
    s = client_ctx.wrap_socket(s, server_hostname="micropython.local")
    print(s.getpeercert(True).hex())
    s.write(b"client to server")
    print(s.read(16))
    s.close()



# test [0,1,-1] edge cases of bignum

long_zero = (2**64) >> 65
long_neg_zero = -long_zero
long_one = long_zero + 1
long_neg_one = -long_one

cases = [long_zero, long_neg_zero, long_one, long_neg_one]

print(cases)
print([-c for c in cases])
print([~c for c in cases])
print([c >> 1 for c in cases])
print([c << 1 for c in cases])

# comparison of 0
print(long_zero == 0)
print(long_neg_zero == 0)
print(long_one - 1 == 0)
print(long_neg_one + 1 == 0)
print(long_zero < 1)
print(long_zero < -1)
print(long_zero > 1)
print(long_zero > -1)
print(long_neg_zero < 1)
print(long_neg_zero < -1)
print(long_neg_zero > 1)
print(long_neg_zero > -1)

# generate zeros that involve negative numbers
large = 1 << 70
large_plus_one = large + 1
zeros = (
    large - large,
    -large + large,
    large + -large,
    -(large - large),
    large - large_plus_one + 1,
    -large & (large - large),
    -large ^ -large,
    -large * (large - large),
    (large - large) // -large,
    -large // -large_plus_one,
    -(large + large) % large,
    (large + large) % -large,
    -(large + large) % -large,
)
print(zeros)

# compute arithmetic operations that may have problems with -0
# (this checks that -0 is never generated in the zeros tuple)
cases = (0, 1, -1) + zeros
for lhs in cases:
    print("-{} = {}".format(lhs, -lhs))
    print("~{} = {}".format(lhs, ~lhs))
    print("{} >> 1 = {}".format(lhs, lhs >> 1))
    print("{} << 1 = {}".format(lhs, lhs << 1))
    for rhs in cases:
        print("{} == {} = {}".format(lhs, rhs, lhs == rhs))
        print("{} + {} = {}".format(lhs, rhs, lhs + rhs))
        print("{} - {} = {}".format(lhs, rhs, lhs - rhs))
        print("{} * {} = {}".format(lhs, rhs, lhs * rhs))
        print("{} | {} = {}".format(lhs, rhs, lhs | rhs))
        print("{} & {} = {}".format(lhs, rhs, lhs & rhs))
        print("{} ^ {} = {}".format(lhs, rhs, lhs ^ rhs))



import ffi
import os
import ffilib


libc = ffilib.libc()

fcntl_l = libc.func("i", "fcntl", "iil")
fcntl_s = libc.func("i", "fcntl", "iip")
ioctl_l = libc.func("i", "ioctl", "iil")
ioctl_s = libc.func("i", "ioctl", "iip")


def fcntl(fd, op, arg=0):
    if type(arg) is int:
        r = fcntl_l(fd, op, arg)
        os.check_error(r)
        return r
    else:
        r = fcntl_s(fd, op, arg)
        os.check_error(r)
        # TODO: Not compliant. CPython says that arg should be immutable,
        # and possibly mutated buffer is returned.
        return r


def ioctl(fd, op, arg=0, mut=False):
    if type(arg) is int:
        r = ioctl_l(fd, op, arg)
        os.check_error(r)
        return r
    else:
        # TODO
        assert mut
        r = ioctl_s(fd, op, arg)
        os.check_error(r)
        return r



# Case of terminating subgen using return with value
def gen():
    yield 1
    yield 2
    return 3

def gen2():
    print("here1")
    print((yield from gen()))
    print("here2")

g = gen2()
print(list(g))


# StopIteration from within a Python function, within a native iterator (map), within a yield from
def gen7(x):
    if x < 3:
        return x
    else:
        raise StopIteration(444)

def gen8():
    print((yield from map(gen7, range(100))))

g = gen8()
print(list(g))



import os

r, w = os.pipe()
print(r, w)

pid = os.fork()
print("hello", pid)

if not pid:
    print("pid:", os.getpid())
    os.close(r)
    os.write(w, "from child")
    os._exit(0)
else:
    os.close(w)
    print(os.read(r, 100))
    print(os.waitpid(pid, 0))



# Test machine.UART.IRQ_RXIDLE firing after a set of characters are received.
#
# IMPORTANT: This test requires hardware connections: the UART TX and RX
# pins must be wired together.

try:
    from machine import UART

    UART.IRQ_RXIDLE
except (ImportError, AttributeError):
    print("SKIP")
    raise SystemExit

import time, sys

# Configure pins based on the target.
if "esp32" in sys.platform:
    uart_id = 1
    tx_pin = 4
    rx_pin = 5
elif "mimxrt" in sys.platform:
    uart_id = 1
    tx_pin = None
elif "pyboard" in sys.platform:
    uart_id = 4
    tx_pin = None  # PA0
    rx_pin = None  # PA1
elif "renesas-ra" in sys.platform:
    uart_id = 9
    tx_pin = None  # P602 @ RA6M2
    rx_pin = None  # P601 @ RA6M2
elif "rp2" in sys.platform:
    uart_id = 0
    tx_pin = "GPIO0"
    rx_pin = "GPIO1"
elif "samd" in sys.platform and "ItsyBitsy M0" in sys.implementation._machine:
    uart_id = 0
    tx_pin = "D1"
    rx_pin = "D0"
    byte_by_byte = True
elif "samd" in sys.platform and "ItsyBitsy M4" in sys.implementation._machine:
    uart_id = 3
    tx_pin = "D1"
    rx_pin = "D0"
else:
    print("Please add support for this test on this platform.")
    raise SystemExit


def irq(u):
    print("IRQ_RXIDLE:", bool(u.irq().flags() & u.IRQ_RXIDLE), "data:", u.read())


text = "12345678"

# Test that the IRQ is called for each set of byte received.
for bits_per_s in (2400, 9600, 115200):
    if tx_pin is None:
        uart = UART(uart_id, bits_per_s)
    else:
        uart = UART(uart_id, bits_per_s, tx=tx_pin, rx=rx_pin)

    uart.irq(irq, uart.IRQ_RXIDLE)

    print("write", bits_per_s)
    uart.write(text)
    uart.flush()
    print("ready")
    time.sleep_ms(100)
    print("done")



# NOTE: Modified version to align with implemented I2C API in nrf port.
#
# Examples usage of SSD1306_SPI on pca10040
#
# from machine import Pin, SPI
# from ssd1306 import SSD1306_SPI
# spi = SPI(0, baudrate=40000000)
# dc = Pin.board.PA11
# res = Pin.board.PA12
# cs = Pin.board.PA13
# disp = SSD1306_SPI(128, 64, spi, dc, res, cs)
#
#
# Example usage of SSD1306_I2C on pca10040
#
# from machine import Pin, I2C
# from ssd1306_mod import SSD1306_I2C_Mod
# i2c = I2C(0, Pin.board.PA3, Pin.board.PA4)
# disp = SSD1306_I2C_Mod(128, 64, i2c)

from ssd1306 import SSD1306_I2C
from micropython import const

SET_COL_ADDR = const(0x21)
SET_PAGE_ADDR = const(0x22)


class SSD1306_I2C_Mod(SSD1306_I2C):
    def show(self):
        x0 = 0
        x1 = self.width - 1
        if self.width == 64:
            # displays with width of 64 pixels are shifted by 32
            x0 += 32
            x1 += 32
        self.write_cmd(SET_COL_ADDR)
        self.write_cmd(x0)
        self.write_cmd(x1)
        self.write_cmd(SET_PAGE_ADDR)
        self.write_cmd(0)
        self.write_cmd(self.pages - 1)

        chunk_size = 254  # 255, excluding opcode.
        num_of_chunks = len(self.buffer) // chunk_size
        leftover = len(self.buffer) - (num_of_chunks * chunk_size)

        for i in range(0, num_of_chunks):
            self.write_data(self.buffer[chunk_size * i : chunk_size * (i + 1)])
        if leftover > 0:
            self.write_data(self.buffer[chunk_size * num_of_chunks :])

    def write_data(self, buf):
        buffer = bytearray([0x40]) + buf  # Co=0, D/C#=1
        self.i2c.writeto(self.addr, buffer)



# Test for VfsFat using a RAM device, mtime feature

try:
    import time, os, vfs

    time.time
    time.sleep
    vfs.VfsFat
except (ImportError, AttributeError):
    print("SKIP")
    raise SystemExit


class RAMBlockDevice:
    ERASE_BLOCK_SIZE = 512

    def __init__(self, blocks):
        self.data = bytearray(blocks * self.ERASE_BLOCK_SIZE)

    def readblocks(self, block, buf):
        addr = block * self.ERASE_BLOCK_SIZE
        for i in range(len(buf)):
            buf[i] = self.data[addr + i]

    def writeblocks(self, block, buf):
        addr = block * self.ERASE_BLOCK_SIZE
        for i in range(len(buf)):
            self.data[addr + i] = buf[i]

    def ioctl(self, op, arg):
        if op == 4:  # block count
            return len(self.data) // self.ERASE_BLOCK_SIZE
        if op == 5:  # block size
            return self.ERASE_BLOCK_SIZE


def test(bdev, vfs_class):
    print("test", vfs_class)

    # Initial format of block device.
    vfs_class.mkfs(bdev)

    # construction
    fs = vfs_class(bdev)

    # Create an empty file, should have a timestamp.
    current_time = int(time.time())
    fs.open("test1", "wt").close()

    # Wait 2 seconds so mtime will increase (FAT has 2 second resolution).
    time.sleep(2)

    # Create another empty file, should have a timestamp.
    fs.open("test2", "wt").close()

    # Stat the files and check mtime is non-zero.
    stat1 = fs.stat("test1")
    stat2 = fs.stat("test2")
    print(stat1[8] != 0, stat2[8] != 0)

    # Check that test1 has mtime which matches time.time() at point of creation.
    # TODO this currently fails on the unix port because FAT stores timestamps
    # in localtime and stat() is UTC based.
    # print(current_time - 1 <= stat1[8] <= current_time + 1)

    # Check that test1 is older than test2.
    print(stat1[8] < stat2[8])

    # Unmount.
    fs.umount()


bdev = RAMBlockDevice(50)
test(bdev, vfs.VfsFat)



import pyb


def test_irq():
    # test basic disable/enable
    i1 = pyb.disable_irq()
    print(i1)
    pyb.enable_irq()  # by default should enable IRQ

    # check that interrupts are enabled by waiting for ticks
    pyb.delay(10)

    # check nested disable/enable
    i1 = pyb.disable_irq()
    i2 = pyb.disable_irq()
    print(i1, i2)
    pyb.enable_irq(i2)
    pyb.enable_irq(i1)

    # check that interrupts are enabled by waiting for ticks
    pyb.delay(10)


test_irq()



import sys
import ffilib
import uctypes


sq3 = ffilib.open("libsqlite3")

# int sqlite3_open(
#  const char *filename,   /* Database filename (UTF-8) */
#  sqlite3 **ppDb          /* OUT: SQLite db handle */
# );
sqlite3_open = sq3.func("i", "sqlite3_open", "sp")
# int sqlite3_config(int, ...);
sqlite3_config = sq3.func("i", "sqlite3_config", "ii")
# int sqlite3_get_autocommit(sqlite3*);
sqlite3_get_autocommit = sq3.func("i", "sqlite3_get_autocommit", "p")
# int sqlite3_close_v2(sqlite3*);
sqlite3_close = sq3.func("i", "sqlite3_close_v2", "p")
# int sqlite3_prepare(
#  sqlite3 *db,            /* Database handle */
#  const char *zSql,       /* SQL statement, UTF-8 encoded */
#  int nByte,              /* Maximum length of zSql in bytes. */
#  sqlite3_stmt **ppStmt,  /* OUT: Statement handle */
#  const char **pzTail     /* OUT: Pointer to unused portion of zSql */
# );
sqlite3_prepare = sq3.func("i", "sqlite3_prepare_v2", "psipp")
# int sqlite3_finalize(sqlite3_stmt *pStmt);
sqlite3_finalize = sq3.func("i", "sqlite3_finalize", "p")
# int sqlite3_step(sqlite3_stmt*);
sqlite3_step = sq3.func("i", "sqlite3_step", "p")
# int sqlite3_column_count(sqlite3_stmt *pStmt);
sqlite3_column_count = sq3.func("i", "sqlite3_column_count", "p")
# int sqlite3_column_type(sqlite3_stmt*, int iCol);
sqlite3_column_type = sq3.func("i", "sqlite3_column_type", "pi")
# int sqlite3_column_int(sqlite3_stmt*, int iCol);
sqlite3_column_int = sq3.func("i", "sqlite3_column_int", "pi")
# double sqlite3_column_double(sqlite3_stmt*, int iCol);
sqlite3_column_double = sq3.func("d", "sqlite3_column_double", "pi")
# const unsigned char *sqlite3_column_text(sqlite3_stmt*, int iCol);
sqlite3_column_text = sq3.func("s", "sqlite3_column_text", "pi")
# sqlite3_int64 sqlite3_last_insert_rowid(sqlite3*);
sqlite3_last_insert_rowid = sq3.func("l", "sqlite3_last_insert_rowid", "p")
# const char *sqlite3_errmsg(sqlite3*);
sqlite3_errmsg = sq3.func("s", "sqlite3_errmsg", "p")


SQLITE_OK = 0
SQLITE_ERROR = 1
SQLITE_BUSY = 5
SQLITE_MISUSE = 21
SQLITE_ROW = 100
SQLITE_DONE = 101

SQLITE_INTEGER = 1
SQLITE_FLOAT = 2
SQLITE_TEXT = 3
SQLITE_BLOB = 4
SQLITE_NULL = 5

SQLITE_CONFIG_URI = 17

# For compatibility with CPython sqlite3 driver
LEGACY_TRANSACTION_CONTROL = -1


class Error(Exception):
    pass


def check_error(db, s):
    if s != SQLITE_OK:
        raise Error(s, sqlite3_errmsg(db))


def get_ptr_size():
    return uctypes.sizeof({"ptr": (0 | uctypes.PTR, uctypes.PTR)})


def __prepare_stmt(db, sql):
    # Prepares a statement
    stmt_ptr = bytes(get_ptr_size())
    res = sqlite3_prepare(db, sql, -1, stmt_ptr, None)
    check_error(db, res)
    return int.from_bytes(stmt_ptr, sys.byteorder)

def __exec_stmt(db, sql):
    # Prepares, executes, and finalizes a statement
    stmt = __prepare_stmt(db, sql)
    sqlite3_step(stmt)
    res = sqlite3_finalize(stmt)
    check_error(db, res)

def __is_dml(sql):
    # Checks if a sql query is a DML, as these get a BEGIN in LEGACY_TRANSACTION_CONTROL
    for dml in ["INSERT", "DELETE", "UPDATE", "MERGE"]:
        if dml in sql.upper():
            return True
    return False


class Connections:
    def __init__(self, db, isolation_level, autocommit):
        self.db = db
        self.isolation_level = isolation_level
        self.autocommit = autocommit

    def commit(self):
        if self.autocommit == LEGACY_TRANSACTION_CONTROL and not sqlite3_get_autocommit(self.db):
            __exec_stmt(self.db, "COMMIT")
        elif self.autocommit == False:
            __exec_stmt(self.db, "COMMIT")
            __exec_stmt(self.db, "BEGIN")

    def rollback(self):
        if self.autocommit == LEGACY_TRANSACTION_CONTROL and not sqlite3_get_autocommit(self.db):
            __exec_stmt(self.db, "ROLLBACK")
        elif self.autocommit == False:
            __exec_stmt(self.db, "ROLLBACK")
            __exec_stmt(self.db, "BEGIN")

    def cursor(self):
        return Cursor(self.db, self.isolation_level, self.autocommit)

    def close(self):
        if self.db:
            if self.autocommit == False and not sqlite3_get_autocommit(self.db):
                __exec_stmt(self.db, "ROLLBACK")

            res = sqlite3_close(self.db)
            check_error(self.db, res)
            self.db = None


class Cursor:
    def __init__(self, db, isolation_level, autocommit):
        self.db = db
        self.isolation_level = isolation_level
        self.autocommit = autocommit
        self.stmt = None

    def __quote(val):
        if isinstance(val, str):
            return "'%s'" % val
        return str(val)

    def execute(self, sql, params=None):
        if self.stmt:
            # If there is an existing statement, finalize that to free it
            res = sqlite3_finalize(self.stmt)
            check_error(self.db, res)

        if params:
            params = [self.__quote(v) for v in params]
            sql = sql % tuple(params)

        if __is_dml(sql) and self.autocommit == LEGACY_TRANSACTION_CONTROL and sqlite3_get_autocommit(self.db):
            # For compatibility with CPython, add functionality for their default transaction
            # behavior. Changing autocommit from LEGACY_TRANSACTION_CONTROL will remove this
            __exec_stmt(self.db, "BEGIN " + self.isolation_level)

        self.stmt = __prepare_stmt(self.db, sql)
        self.num_cols = sqlite3_column_count(self.stmt)

        if not self.num_cols:
            v = self.fetchone()
            # If it's not select, actually execute it here
            # num_cols == 0 for statements which don't return data (=> modify it)
            assert v is None
            self.lastrowid = sqlite3_last_insert_rowid(self.db)

    def close(self):
        if self.stmt:
            res = sqlite3_finalize(self.stmt)
            check_error(self.db, res)
            self.stmt = None

    def __make_row(self):
        res = []
        for i in range(self.num_cols):
            t = sqlite3_column_type(self.stmt, i)
            if t == SQLITE_INTEGER:
                res.append(sqlite3_column_int(self.stmt, i))
            elif t == SQLITE_FLOAT:
                res.append(sqlite3_column_double(self.stmt, i))
            elif t == SQLITE_TEXT:
                res.append(sqlite3_column_text(self.stmt, i))
            else:
                raise NotImplementedError
        return tuple(res)

    def fetchone(self):
        res = sqlite3_step(self.stmt)
        if res == SQLITE_DONE:
            return None
        if res == SQLITE_ROW:
            return self.__make_row()
        check_error(self.db, res)


def connect(fname, uri=False, isolation_level="", autocommit=LEGACY_TRANSACTION_CONTROL):
    if isolation_level not in [None, "", "DEFERRED", "IMMEDIATE", "EXCLUSIVE"]:
        raise Error("Invalid option for isolation level")

    sqlite3_config(SQLITE_CONFIG_URI, int(uri))

    sqlite_ptr = bytes(get_ptr_size())
    sqlite3_open(fname, sqlite_ptr)
    db = int.from_bytes(sqlite_ptr, sys.byteorder)

    if autocommit == False:
        __exec_stmt(db, "BEGIN")

    return Connections(db, isolation_level, autocommit)



try:
    import micropython
except:
    pass


def mandelbrot():
    # returns True if c, complex, is in the Mandelbrot set
    # @micropython.native
    def in_set(c):
        z = 0
        for i in range(40):
            z = z * z + c
            if abs(z) > 60:
                return False
        return True

    lcd.clear()
    for u in range(91):
        for v in range(31):
            if in_set((u / 30 - 2) + (v / 15 - 1) * 1j):
                lcd.set(u, v)
    lcd.show()


# PC testing
import lcd

lcd = lcd.LCD(128, 32)
mandelbrot()



# test capability to start a thread with keyword args
#
# MIT license; Copyright (c) 2016 Damien P. George on behalf of Pycom Ltd

import time
import _thread


def thread_entry(a0, a1, a2, a3):
    print("thread", a0, a1, a2, a3)


# spawn thread using kw args
_thread.start_new_thread(thread_entry, (10, 20), {"a2": 0, "a3": 1})

# wait for thread to finish
time.sleep(1)

# incorrect argument where dictionary is needed for keyword args
try:
    _thread.start_new_thread(thread_entry, (), ())
except TypeError:
    print("TypeError")

print("done")



# tests meminfo functions in micropython module

import micropython

# these functions are not always available
if not hasattr(micropython, "mem_total"):
    print("SKIP")
else:
    t = micropython.mem_total()
    c = micropython.mem_current()
    p = micropython.mem_peak()

    l = list(range(10000))

    print(micropython.mem_total() > t)
    print(micropython.mem_current() > c)
    print(micropython.mem_peak() > p)



# MIT license; Copyright (c) 2023 Jim Mussared
# Originally ported from CPython by Paul Sokolovsky

from ._sha512 import sha512


class sha384(sha512):
    digest_size = digestsize = 48
    _iv = [
        0xCBBB9D5DC1059ED8,
        0x629A292A367CD507,
        0x9159015A3070DD17,
        0x152FECD8F70E5939,
        0x67332667FFC00B31,
        0x8EB44A8768581511,
        0xDB0C2E0D64F98FA7,
        0x47B5481DBEFA4FA4,
    ]



# Test creating an SSL connection with specified ciphers.

try:
    import os
    import socket
    import tls
except ImportError:
    print("SKIP")
    raise SystemExit

PORT = 8000

# These are test certificates. See tests/README.md for details.
cert = cafile = "ec_cert.der"
key = "ec_key.der"

try:
    with open(cafile, "rb") as f:
        cadata = f.read()
    with open(key, "rb") as f:
        keydata = f.read()
except OSError:
    print("SKIP")
    raise SystemExit


# Server
def instance0():
    multitest.globals(IP=multitest.get_network_ip())
    s = socket.socket()
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.bind(socket.getaddrinfo("0.0.0.0", PORT)[0][-1])
    s.listen(1)
    multitest.next()
    s2, _ = s.accept()
    server_ctx = tls.SSLContext(tls.PROTOCOL_TLS_SERVER)
    server_ctx.load_cert_chain(cadata, keydata)
    s2 = server_ctx.wrap_socket(s2, server_side=True)
    assert isinstance(s2.cipher(), tuple)
    print(s2.read(16))
    s2.write(b"server to client")
    s2.close()
    s.close()


# Client
def instance1():
    multitest.next()
    s = socket.socket()
    s.connect(socket.getaddrinfo(IP, PORT)[0][-1])
    client_ctx = tls.SSLContext(tls.PROTOCOL_TLS_CLIENT)
    ciphers = client_ctx.get_ciphers()
    assert "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256" in ciphers
    client_ctx.set_ciphers(["TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256"])
    client_ctx.verify_mode = tls.CERT_REQUIRED
    client_ctx.load_verify_locations(cadata)
    s = client_ctx.wrap_socket(s, server_hostname="micropython.local")
    s.write(b"client to server")
    print(s.read(16))
    s.close()



# Test SSL ioctl method.
# Direct access to this method is only available if MICROPY_UNIX_COVERAGE is enabled.

try:
    import io, ssl

    io.BytesIO
except (ImportError, AttributeError):
    print("SKIP")
    raise SystemExit

_MP_STREAM_POLL = 3
_MP_STREAM_CLOSE = 4
_MP_STREAM_GET_FILENO = 10

s = ssl.wrap_socket(io.BytesIO(), server_side=1, do_handshake=0)

if not hasattr(s, "ioctl"):
    print("SKIP")
    raise SystemExit

# These ioctl's should be unsupported.
for request in (-1, 0, _MP_STREAM_GET_FILENO):
    try:
        s.ioctl(request, 0)
    except OSError:
        print(request, "OSError")

# These ioctl's should be supported.
for request in (_MP_STREAM_CLOSE, _MP_STREAM_POLL, _MP_STREAM_CLOSE):
    print(request, s.ioctl(request, 0))



import os
import socket
import ssl

# This certificate was obtained from micropython.org using openssl:
# $ openssl s_client -showcerts -connect micropython.org:443 </dev/null 2>/dev/null
# The certificate is from Let's Encrypt:
# 1 s:C=US, O=Let's Encrypt, CN=R11
#   i:C=US, O=Internet Security Research Group, CN=ISRG Root X1
#   a:PKEY: rsaEncryption, 2048 (bit); sigalg: RSA-SHA256
#   v:NotBefore: Mar 13 00:00:00 2024 GMT; NotAfter: Mar 12 23:59:59 2027 GMT
# Copy PEM content to a file (certmpy.pem) and convert to DER e.g.
# $ openssl x509 -in certmpy.pem -out certmpy.der -outform DER
# Then convert to hex format, eg using binascii.hexlify(data).


ca_cert_chain = "mpycert.der"
try:
    os.stat(ca_cert_chain)
except OSError:
    print("SKIP")
    raise SystemExit


def main(use_stream=True):
    context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)

    context.verify_mode = ssl.CERT_REQUIRED
    assert context.verify_mode == ssl.CERT_REQUIRED

    context.load_verify_locations(cafile=ca_cert_chain)

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    addr = socket.getaddrinfo("micropython.org", 443)[0][-1]

    # CPython can wrap the socket even if not connected yet.
    # ssl_sock = context.wrap_socket(s, server_hostname='micropython.org')
    # ssl_sock.connect(addr)

    # MicroPython needs to connect first, CPython can do this too.
    s.connect(addr)
    # server_hostname must match CN (Common Name) in the certificate
    # presented by the server
    ssl_sock = context.wrap_socket(s, server_hostname="micropython.org")
    ssl_sock.write(b"GET / HTTP/1.0\r\n\r\n")
    print(ssl_sock.read(17))
    assert isinstance(ssl_sock.cipher(), tuple)
    ssl_sock.close()


main()



# default separator (whitespace)
print(b"a b".split())
print(b"   a   b    ".split(None))
print(b"   a   b    ".split(None, 1))
print(b"   a   b    ".split(None, 2))
print(b"   a   b  c  ".split(None, 1))
print(b"   a   b  c  ".split(None, 0))
print(b"   a   b  c  ".split(None, -1))

# empty separator should fail
try:
    b"abc".split(b'')
except ValueError:
    print("ValueError")

# non-empty separator
print(b"abc".split(b"a"))
print(b"abc".split(b"b"))
print(b"abc".split(b"c"))
print(b"abc".split(b"z"))
print(b"abc".split(b"ab"))
print(b"abc".split(b"bc"))
print(b"abc".split(b"abc"))
print(b"abc".split(b"abcd"))
print(b"abcabc".split(b"bc"))
print(b"abcabc".split(b"bc", 0))
print(b"abcabc".split(b"bc", 1))
print(b"abcabc".split(b"bc", 2))



import ssl
import os
import asyncio

# This certificate was obtained from micropython.org using openssl:
# $ openssl s_client -showcerts -connect micropython.org:443 </dev/null 2>/dev/null
# The certificate is from Let's Encrypt:
# 1 s:/C=US/O=Let's Encrypt/CN=R3
#   i:/C=US/O=Internet Security Research Group/CN=ISRG Root X1
# Validity
#            Not Before: Sep  4 00:00:00 2020 GMT
#            Not After : Sep 15 16:00:00 2025 GMT
# Copy PEM content to a file (certmpy.pem) and convert to DER e.g.
# $ openssl x509 -in certmpy.pem -out certmpy.der -outform DER
# Then convert to hex format, eg using binascii.hexlify(data).

# Note that the instructions above is to obtain an intermediate
# root CA cert that works for MicroPython. However CPython needs the ultimate root CA
# cert from ISRG

ca_cert_chain = "isrg.der"

try:
    os.stat(ca_cert_chain)
except OSError:
    print("SKIP")
    raise SystemExit

with open(ca_cert_chain, "rb") as ca:
    cadata = ca.read()

client_ctx = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
client_ctx.verify_mode = ssl.CERT_REQUIRED
client_ctx.load_verify_locations(cadata=cadata)


async def http_get(url, port, sslctx):
    reader, writer = await asyncio.open_connection(url, port, ssl=sslctx)

    print("write GET")
    writer.write(b"GET / HTTP/1.0\r\n\r\n")
    await writer.drain()

    print("read response")
    while True:
        data = await reader.readline()
        # avoid printing datetime which makes the test fail
        if b"GMT" not in data:
            print("read:", data)
        if not data:
            break

    print("close")
    writer.close()
    await writer.wait_closed()
    print("done")


asyncio.run(http_get("micropython.org", 443, client_ctx))



# This test is mostly intended for ensuring USB serial stays stable over
# lightsleep, but can double as a general lightsleep test.
#
# In theory this should run on any port, but native USB REPL doesn't currently
# recover automatically on all ports. On pyboard and ESP32-S3 the host needs to
# send something to the port before it responds again. Possibly the same for other
# ports with native USB.
#
# A range of sleep periods (1 to 512ms) are tested. The total nominal sleep time
# is 10.23 seconds, but on most ports this will finish much earlier as interrupts
# happen before each timeout expires.
import sys

try:
    from machine import lightsleep, Pin
except ImportError:
    print("SKIP")
    raise SystemExit

# RP2350 currently fails this test, needs further investigation.
if "RP2350" in sys.implementation._machine:
    print("SKIP")
    raise SystemExit

try:
    led = Pin(Pin.board.LED, Pin.OUT)
except AttributeError:
    led = None

for n in range(100):
    if led:
        led.toggle()
    sys.stdout.write(chr(ord("a") + (n % 26)))
    lightsleep(2 ** (n % 10))

print("\nDONE")



import sys

# ruff: noqa: E402
sys.path.insert(0, ".")
import aiohttp
import asyncio


headers = {"Authorization": "Basic bG9naW46cGFzcw=="}


async def main():
    async with aiohttp.ClientSession(headers=headers) as session:
        async with session.get("http://httpbin.org/headers") as r:
            json_body = await r.json()
            print(json_body)


asyncio.run(main())



# This module is not entirely compatible with CPython
import os


os.putenv("TEST_VARIABLE", "TEST_VALUE")

print(os.getenv("TEST_VARIABLE"))
print(os.getenv("TEST_VARIABLE", "TEST_DEFAULT_VALUE"))

os.unsetenv("TEST_VARIABLE")

print(os.getenv("TEST_VARIABLE"))
print(os.getenv("TEST_VARIABLE", "TEST_DEFAULT_VALUE"))

print(os.system("exit 0"))

rand = os.urandom(4)
print(type(rand) is bytes, len(rand))

os.errno(2)
print(os.errno())



# aioespnow module for MicroPython on ESP32 and ESP8266
# MIT license; Copyright (c) 2022 Glenn Moloney @glenn20

import asyncio
import espnow


# Modelled on the asyncio.Stream class (extmod/asyncio/stream.py)
# NOTE: Relies on internal implementation of asyncio.core (_io_queue)
class AIOESPNow(espnow.ESPNow):
    # Read one ESPNow message
    async def arecv(self):
        yield asyncio.core._io_queue.queue_read(self)
        return self.recv(0)  # type: ignore[misc]

    async def airecv(self):
        yield asyncio.core._io_queue.queue_read(self)
        return self.irecv(0)  # type: ignore[misc]

    async def asend(self, mac, msg=None, sync=None):
        if msg is None:
            msg, mac = mac, None  # If msg is None: swap mac and msg
        yield asyncio.core._io_queue.queue_write(self)
        return self.send(mac, msg, sync)  # type: ignore[misc]

    # "async for" support
    def __aiter__(self):
        return self

    async def __anext__(self):
        return await self.airecv()



# create a class that has a __getitem__ method
class A:
    def __getitem__(self, index):
        print('getitem', index)
        if index > 10:
            raise StopIteration

# test __getitem__
A()[0]
A()[1]

# iterate using a for loop
for i in A():
    pass

# iterate manually
it = iter(A())
try:
    while True:
        next(it)
except StopIteration:
    pass

# this class raises an IndexError to stop the iteration
class A:
    def __getitem__(self, i):
        raise IndexError
print(list(A()))

# this class raises a non-StopIteration exception on iteration
class A:
    def __getitem__(self, i):
        raise TypeError
try:
    for i in A():
        pass
except TypeError:
    print("TypeError")



import time
import unittest

DAYS = ("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")

MONTHS = (
    "January",
    "February",
    "March",
    "April",
    "May",
    "June",
    "July",
    "August",
    "September",
    "October",
    "November",
    "December",
)

TIME_TUPLE = (2022, 12, 14, 0, 45, 17, 2, 348, 0)


class TestStrftime(unittest.TestCase):
    def test_not_formatting(self):
        fmt = "a string with no formatting {}[]() 0123456789 !@#$^&*"
        self.assertEqual(time.strftime(fmt, TIME_TUPLE), fmt)

    def test_days(self):
        for i, day in enumerate(DAYS):
            t = (0, 0, 0, 0, 0, 0, i, 0, 0)
            self.assertEqual(time.strftime("%a%A", t), day[:3] + day)

    def test_months(self):
        for i, month in enumerate(MONTHS):
            t = (0, i + 1, 0, 0, 0, 0, 0, 0, 0)
            self.assertEqual(time.strftime("%b%B", t), month[:3] + month)

    def test_full(self):
        fmt = "%Y-%m-%d %a %b %I:%M:%S %%%P%%"
        expected = "2022-12-14 Wed Dec 00:45:17 %AM%"
        self.assertEqual(time.strftime(fmt, TIME_TUPLE), expected)



# FeatherS3 Neo MicroPython Helper Library
# MIT license; Copyright (c) 2024 Seon Rozenblum - Unexpected Maker
#
# Project home:
# http://esp32s3.com

# Import required libraries
from micropython import const
from machine import Pin, ADC
import time

# FeatherS3 Hardware Pin Assignments

# Sense Pins
VBUS_SENSE = const(15)
VBAT_SENSE = const(2)

# LDO2 & Other Pins
LDO2 = const(39)
LED = const(13)
AMB_LIGHT = const(4)

# RGB LED Pins
RGB_DATA = const(40)
RGB_PWR = const(39)

# RGB MATRIX LED Pins
RGB_MATRIX_DATA = const(16)
RGB_MATRIX_PWR = const(39)

# SPI
SPI_MOSI = const(35)
SPI_MISO = const(37)
SPI_CLK = const(36)

# I2C
I2C_SDA = const(8)
I2C_SCL = const(9)

# Helper functions


# LED & Ambient Light Sensor control
def led_set(state):
    """Set the state of the BLUE LED on IO13"""
    l = Pin(LED, Pin.OUT)
    l.value(state)


def led_blink():
    """Toggle the BLUE LED on IO13"""
    l = Pin(LED, Pin.OUT)
    l.value(not l.value())


# Create ADC and set attenuation and return the ambient light value from the onboard sensor
def get_amb_light():
    """Get Ambient Light Sensor reading"""
    adc = ADC(Pin(AMB_LIGHT))
    adc.atten(ADC.ATTN_11DB)
    return adc.read()


def set_ldo2_power(state):
    """
    Enable or Disable power to the second LDO, which is the LDO that powers the following items
    RGB Matrix, RGB status LED, Ambient light Sensor.
    This is ON by default and will automatically shut down when the ESP32 going into deep sleep.
    """
    Pin(LDO2, Pin.OUT).value(state)


def get_battery_voltage():
    """
    Returns the current battery voltage. If no battery is connected, returns 4.2V which is the charge voltage
    This is an approximation only, but useful to detect if the charge state of the battery is getting low.
    """
    adc = ADC(Pin(VBAT_SENSE))  # Assign the ADC pin to read
    # Max voltage on ADC input will be 4.2V divided by R2 (442K) & R5 (160K), 4.2 / (160+442) * 160 = 1.1163V
    adc.atten(ADC.ATTN_2_5DB)  # Needs 2.5DB attenuation to read a max voltage of 1.1163V
    # Use read_uv() to get ADC reading as this will use the on-chip calibration data
    measuredvbat = adc.read_uv() / 1000000  # Read micovolts and convert to volts
    measuredvbat *= 3.7624  # Multiply by ratio of battery voltage to ADC pin voltage: 4.2/1.1163
    return round(measuredvbat, 2)


def get_vbus_present():
    """Detect if VBUS (5V) power source is present"""
    return Pin(VBUS_SENSE, Pin.IN).value() == 1


# NeoPixel rainbow colour wheel
def rgb_color_wheel(wheel_pos):
    """Color wheel to allow for cycling through the rainbow of RGB colors."""
    wheel_pos = wheel_pos % 255

    if wheel_pos < 85:
        return 255 - wheel_pos * 3, 0, wheel_pos * 3
    elif wheel_pos < 170:
        wheel_pos -= 85
        return 0, wheel_pos * 3, 255 - wheel_pos * 3
    else:
        wheel_pos -= 170
        return wheel_pos * 3, 255 - wheel_pos * 3, 0



# test native try handling


# deeply nested try (9 deep)
@micropython.native
def f():
    try:
        try:
            try:
                try:
                    try:
                        try:
                            try:
                                try:
                                    try:
                                        raise ValueError
                                    finally:
                                        print(8)
                                finally:
                                    print(7)
                            finally:
                                print(6)
                        finally:
                            print(5)
                    finally:
                        print(4)
                finally:
                    print(3)
            finally:
                print(2)
        finally:
            print(1)
    except ValueError:
        print("ValueError")


f()



#!/usr/bin/env python


from collections import defaultdict, namedtuple
import os
import re
import sys

sys.path.insert(0, os.path.join(os.path.dirname(__file__), "../../../tools"))
import boardgen


# These are the columns of the af.csv as well as the arguments to the PIN()
# macro in samd_prefix.c.
AFS = {
    "SAMD21": ["eic", "adc0", "sercom1", "sercom2", "tcc1", "tcc2"],
    "SAMD51": ["eic", "adc0", "adc1", "sercom1", "sercom2", "tc", "tcc1", "tcc2"],
}


class SamdPin(boardgen.Pin):
    def __init__(self, cpu_pin_name):
        super().__init__(cpu_pin_name)

        # P<port><num> (already verified by validate_cpu_pin_name).
        self._port = cpu_pin_name[1]
        self._pin = int(cpu_pin_name[2:])

        # List of uint8 values from the af.csv. Default to 0xff if not
        # present.
        self._afs = defaultdict(lambda: 0xFF)

    # Called for each AF defined in the csv file for this pin.
    def add_af(self, af_idx, af_name, af):
        # samd-specific: Any pin in the af.csv is implicitly part of the board
        # pins.csv and will therefore be in the Pin.cpu dict. This is
        # equivalent to adding `,Pxy` to the board.csv for every pin in the
        # af.csv.
        self._available = True

        name = AFS[self._generator.args.mcu][af_idx]
        assert name == af_name.lower()
        if name == "eic" or name.startswith("adc"):
            # eic and adc columns are decimal.
            v = int(af)
        else:
            # hex, high nibble is device number, low nibble is pad number.
            v = int(af, 16)
        self._afs[AFS[self._generator.args.mcu][af_idx]] = v

    # Use the PIN() macro defined in samd_prefix.c for defining the pin
    # objects.
    def definition(self):
        # SAMD21: PIN(p_name, p_eic, p_adc0, p_sercom1, p_sercom2, p_tcc1, p_tcc2)
        # SAMD51: PIN(p_name, p_eic, p_adc0, p_adc1, p_sercom1, p_sercom2, p_tc, p_tcc1, p_tcc2)
        return "PIN({:s}, {})".format(
            self.name(),
            ", ".join("0x{:02x}".format(self._afs[x]) for x in AFS[self._generator.args.mcu]),
        )

    # Wrap all definitions to ensure that the ASF defines this pin for this
    # particular MCU.
    def enable_macro(self):
        return "defined(PIN_{})".format(self.name())

    # SAMD cpu names must be "P<port><num>", with zero-prefixed two-digit <num>.
    @staticmethod
    def validate_cpu_pin_name(cpu_pin_name):
        boardgen.Pin.validate_cpu_pin_name(cpu_pin_name)

        if not re.match("P[A-D][0-9][0-9]$", cpu_pin_name):
            raise boardgen.PinGeneratorError("Invalid cpu pin name '{}'".format(cpu_pin_name))


class SamdPinGenerator(boardgen.PinGenerator):
    def __init__(self):
        # Use custom pin type above, and also enable the --af-csv argument so
        # that add_af gets called on each pin.
        super().__init__(
            pin_type=SamdPin,
            enable_af=True,
        )

    # Override the default implementation just to change the default arguments
    # (extra header row, skip first column).
    def parse_af_csv(self, filename):
        return super().parse_af_csv(filename, header_rows=1, pin_col=0, af_col=1)

    # We need to know the mcu to emit the correct AF list.
    def extra_args(self, parser):
        parser.add_argument("--mcu")


if __name__ == "__main__":
    SamdPinGenerator().main()



# test binary operations on range objects; (in)equality only

# this "feature test" actually tests the implementation but is the best we can do
if range(1) != range(1):
    print("SKIP")
    raise SystemExit

# basic (in)equality
print(range(1) == range(1))
print(range(1) != range(1))
print(range(1) != range(2))

# empty range
print(range(0) == range(0))
print(range(1, 0) == range(0))
print(range(1, 4, -1) == range(6, 3))

# 1 element range
print(range(1, 4, 10) == range(1, 4, 10))
print(range(1, 4, 10) == range(1, 4, 20))
print(range(1, 4, 10) == range(1, 8, 20))

# more than 1 element
print(range(0, 3, 2) == range(0, 3, 2))
print(range(0, 3, 2) == range(0, 4, 2))
print(range(0, 3, 2) == range(0, 5, 2))

# unsupported binary op
try:
    range(1) + 10
except TypeError:
    print('TypeError')



# Test constant optimisation, with full range of const types.
# This test will only work when MICROPY_COMP_CONST and MICROPY_COMP_CONST_TUPLE are enabled.

from micropython import const

_INT = const(123)
_STR = const("str")
_BYTES = const(b"bytes")
_TUPLE = const((_INT, _STR, _BYTES))
_TUPLE2 = const((None, False, True, ..., (), _TUPLE))

print(_INT)
print(_STR)
print(_BYTES)
print(_TUPLE)
print(_TUPLE2)

x = _TUPLE
print(x is _TUPLE)
print(x is (_INT, _STR, _BYTES))

print(hasattr(globals(), "_INT"))
print(hasattr(globals(), "_STR"))
print(hasattr(globals(), "_BYTES"))
print(hasattr(globals(), "_TUPLE"))
print(hasattr(globals(), "_TUPLE2"))



def attrgetter(attr):
    assert "." not in attr

    def _attrgetter(obj):
        return getattr(obj, attr)

    return _attrgetter


def lt(a, b):
    return a < b


def le(a, b):
    return a <= b


def gt(a, b):
    return a > b


def ge(a, b):
    return a >= b


def eq(a, b):
    return a == b


def ne(a, b):
    return a != b


def mod(a, b):
    return a % b


def truediv(a, b):
    return a / b


def floordiv(a, b):
    return a // b



try:
    import framebuf
except ImportError:
    print("SKIP")
    raise SystemExit


def printbuf():
    print("--8<--")
    bytes_per_row = w // 2
    for y in range(h):
        for x in range(bytes_per_row):
            print("%02x" % buf[(x + y * bytes_per_row)], end="")
        print()
    print("-->8--")


w = 10
h = 10
buf = bytearray(w * h // 2)
fbuf = framebuf.FrameBuffer(buf, w, h, framebuf.GS4_HMSB)


def prepare_buffer():
    fbuf.fill(0)
    fbuf.rect(2, 0, 6, 10, 0x07, True)
    fbuf.rect(0, 2, 10, 6, 0x01, True)


prepare_buffer()
printbuf()

fbuf.scroll(5, -1)
printbuf()

prepare_buffer()
fbuf.scroll(-5, 5)
printbuf()

prepare_buffer()
# Scrolling by at least the size of buffer, no change to buffer.
fbuf.scroll(15, 7)
fbuf.scroll(10, -1)
fbuf.scroll(1, -10)
printbuf()



"""
Generate header file with macros defining MicroPython version info.

This script works with Python 2.6, 2.7, 3.3 and 3.4.
"""

from __future__ import print_function

import argparse
import sys
import os
import datetime
import subprocess


# The MicroPython repository tags a release commit as "vX.Y.Z", and the commit
# immediately following as "vX.(Y+1).Z-preview".
# This function will return:
#   "vX.Y.Z" -- building at the release commit
#   "vX.Y.Z-preview" -- building at the first commit in the next cycle
#   "vX.Y.Z-preview.N.gHASH" -- building at any subsequent commit in the cycle
#   "vX.Y.Z-preview.N.gHASH.dirty" -- building at any subsequent commit in the cycle
#                                     with local changes
def get_version_info_from_git(repo_path):
    # Python 2.6 doesn't have check_output, so check for that
    try:
        subprocess.check_output
    except AttributeError:
        return None

    # Note: git describe doesn't work if no tag is available
    try:
        git_tag = subprocess.check_output(
            ["git", "describe", "--tags", "--dirty", "--always", "--match", "v[1-9].*"],
            cwd=repo_path,
            stderr=subprocess.STDOUT,
            universal_newlines=True,
        ).strip()
        # Turn git-describe's output into semver compatible (dot-separated
        # identifiers inside the prerelease field).
        git_tag = git_tag.split("-", 1)
        if len(git_tag) == 1:
            return git_tag[0]
        else:
            return git_tag[0] + "-" + git_tag[1].replace("-", ".")
    except (subprocess.CalledProcessError, OSError):
        return None


def get_hash_from_git(repo_path):
    # Python 2.6 doesn't have check_output, so check for that.
    try:
        subprocess.check_output
    except AttributeError:
        return None

    try:
        return subprocess.check_output(
            ["git", "rev-parse", "--short", "HEAD"],
            cwd=repo_path,
            stderr=subprocess.STDOUT,
            universal_newlines=True,
        ).strip()
    except (subprocess.CalledProcessError, OSError):
        return None


# When building from a source tarball (or any situation where the git repo
# isn't available), this function will use the info in mpconfig.h as a
# fallback. The release commit sets MICROPY_VERSION_PRERELEASE to 0, and the
# commit immediately following increments MICROPY_VERSION_MINOR and sets
# MICROPY_VERSION_PRERELEASE back to 1.
# This function will return:
#    "vX.Y.Z" -- building at the release commit
#    "vX.Y.Z-preview" -- building at any other commit
def get_version_info_from_mpconfig(repo_path):
    print(
        "makeversionhdr.py: Warning: No git repo or tag info available, falling back to mpconfig.h version info.",
        file=sys.stderr,
    )

    with open(os.path.join(repo_path, "py", "mpconfig.h")) as f:
        for line in f:
            if line.startswith("#define MICROPY_VERSION_MAJOR "):
                ver_major = int(line.strip().split()[2])
            elif line.startswith("#define MICROPY_VERSION_MINOR "):
                ver_minor = int(line.strip().split()[2])
            elif line.startswith("#define MICROPY_VERSION_MICRO "):
                ver_micro = int(line.strip().split()[2])
            elif line.startswith("#define MICROPY_VERSION_PRERELEASE "):
                ver_prerelease = int(line.strip().split()[2])
                git_tag = "v%d.%d.%d%s" % (
                    ver_major,
                    ver_minor,
                    ver_micro,
                    "-preview" if ver_prerelease else "",
                )
                return git_tag
    return None


def make_version_header(repo_path, filename):
    git_tag = None
    git_hash = None
    if "MICROPY_GIT_TAG" in os.environ:
        git_tag = os.environ["MICROPY_GIT_TAG"]
        git_hash = os.environ.get("MICROPY_GIT_HASH")
    if git_tag is None:
        git_tag = get_version_info_from_git(repo_path)
        git_hash = get_hash_from_git(repo_path)
    if git_tag is None:
        git_tag = get_version_info_from_mpconfig(repo_path)

    if not git_tag:
        print("makeversionhdr.py: Error: No version information available.")
        sys.exit(1)

    build_date = datetime.date.today()
    if "SOURCE_DATE_EPOCH" in os.environ:
        build_date = datetime.datetime.utcfromtimestamp(
            int(os.environ["SOURCE_DATE_EPOCH"])
        ).date()

    # Generate the file with the git and version info
    # Note: MICROPY_GIT_HASH may be used by third-party code.
    file_data = """\
// This file was generated by py/makeversionhdr.py
#define MICROPY_GIT_TAG "%s"
#define MICROPY_GIT_HASH "%s"
#define MICROPY_BUILD_DATE "%s"
""" % (
        git_tag,
        git_hash or "<no hash>",
        build_date.strftime("%Y-%m-%d"),
    )

    # Check if the file contents changed from last time
    write_file = True
    if os.path.isfile(filename):
        with open(filename, "r") as f:
            existing_data = f.read()
        if existing_data == file_data:
            write_file = False

    # Only write the file if we need to
    if write_file:
        print("GEN %s" % filename)
        with open(filename, "w") as f:
            f.write(file_data)


def main():
    parser = argparse.ArgumentParser()
    # makeversionheader.py lives in repo/py, so default repo_path to the
    # parent of sys.argv[0]'s directory.
    parser.add_argument(
        "-r",
        "--repo-path",
        default=os.path.join(os.path.dirname(sys.argv[0]), ".."),
        help="path to MicroPython Git repo to query for version",
    )
    parser.add_argument("dest", nargs=1, help="output file path")
    args = parser.parse_args()

    make_version_header(args.repo_path, args.dest[0])


if __name__ == "__main__":
    main()



# test non-blocking UDP sockets

try:
    import socket, errno
except ImportError:
    print("SKIP")
    raise SystemExit

try:
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.bind(socket.getaddrinfo("127.0.0.1", 8000)[0][-1])
except OSError:
    print("SKIP")
    raise SystemExit

s.settimeout(0)

try:
    s.recv(1)
except OSError as er:
    print("EAGAIN:", er.errno == errno.EAGAIN)



# MicroPython asyncio module
# MIT license; Copyright (c) 2019-2022 Damien P. George

from . import core


async def _run(waiter, aw):
    try:
        result = await aw
        status = True
    except BaseException as er:
        result = None
        status = er
    if waiter.data is None:
        # The waiter is still waiting, cancel it.
        if waiter.cancel():
            # Waiter was cancelled by us, change its CancelledError to an instance of
            # CancelledError that contains the status and result of waiting on aw.
            # If the wait_for task subsequently gets cancelled externally then this
            # instance will be reset to a CancelledError instance without arguments.
            waiter.data = core.CancelledError(status, result)


async def wait_for(aw, timeout, sleep=core.sleep):
    aw = core._promote_to_task(aw)
    if timeout is None:
        return await aw

    # Run aw in a separate runner task that manages its exceptions.
    runner_task = core.create_task(_run(core.cur_task, aw))

    try:
        # Wait for the timeout to elapse.
        await sleep(timeout)
    except core.CancelledError as er:
        status = er.value
        if status is None:
            # This wait_for was cancelled externally, so cancel aw and re-raise.
            runner_task.cancel()
            raise er
        elif status is True:
            # aw completed successfully and cancelled the sleep, so return aw's result.
            return er.args[1]
        else:
            # aw raised an exception, propagate it out to the caller.
            raise status

    # The sleep finished before aw, so cancel aw and raise TimeoutError.
    runner_task.cancel()
    await runner_task
    raise core.TimeoutError


def wait_for_ms(aw, timeout):
    return wait_for(aw, timeout, core.sleep_ms)


class _Remove:
    @staticmethod
    def remove(t):
        pass


# async
def gather(*aws, return_exceptions=False):
    def done(t, er):
        # Sub-task "t" has finished, with exception "er".
        nonlocal state
        if gather_task.data is not _Remove:
            # The main gather task has already been scheduled, so do nothing.
            # This happens if another sub-task already raised an exception and
            # woke the main gather task (via this done function), or if the main
            # gather task was cancelled externally.
            return
        elif not return_exceptions and not isinstance(er, StopIteration):
            # A sub-task raised an exception, indicate that to the gather task.
            state = er
        else:
            state -= 1
            if state:
                # Still some sub-tasks running.
                return
        # Gather waiting is done, schedule the main gather task.
        core._task_queue.push(gather_task)

    # Prepare the sub-tasks for the gather.
    # The `state` variable counts the number of tasks to wait for, and can be negative
    # if the gather should not run at all (because a task already had an exception).
    ts = [core._promote_to_task(aw) for aw in aws]
    state = 0
    for i in range(len(ts)):
        if ts[i].state is True:
            # Task is running, register the callback to call when the task is done.
            ts[i].state = done
            state += 1
        elif not ts[i].state:
            # Task finished already.
            if not isinstance(ts[i].data, StopIteration):
                # Task finished by raising an exception.
                if not return_exceptions:
                    # Do not run this gather at all.
                    state = -len(ts)
        else:
            # Task being waited on, gather not currently supported for this case.
            raise RuntimeError("can't gather")

    # Set the state for execution of the gather.
    gather_task = core.cur_task
    cancel_all = False

    # Wait for a sub-task to need attention (if there are any to wait for).
    if state > 0:
        gather_task.data = _Remove
        try:
            yield
        except core.CancelledError as er:
            cancel_all = True
            state = er

    # Clean up tasks.
    for i in range(len(ts)):
        if ts[i].state is done:
            # Sub-task is still running, deregister the callback and cancel if needed.
            ts[i].state = True
            if cancel_all:
                ts[i].cancel()
        elif isinstance(ts[i].data, StopIteration):
            # Sub-task ran to completion, get its return value.
            ts[i] = ts[i].data.value
        else:
            # Sub-task had an exception.
            if return_exceptions:
                # Get the sub-task exception to return in the list of return values.
                ts[i] = ts[i].data
            elif isinstance(state, int):
                # Raise the sub-task exception, if there is not already an exception to raise.
                state = ts[i].data

    # Either this gather was cancelled, or one of the sub-tasks raised an exception with
    # return_exceptions==False, so reraise the exception here.
    if state:
        raise state

    # Return the list of return values of each sub-task.
    return ts



# MicroPython LoRa async modem driver
# MIT license; Copyright (c) 2023 Angus Gratton
#
# LoRa is a registered trademark or service mark of Semtech Corporation or its affiliates.

import asyncio
import time
from micropython import const

# Set to True to get some additional printed debug output.
_DEBUG = const(False)

# Some "belts and braces" timeouts when using IRQs, to wake up if an IRQ hasn't
# fired as expected. Also a timeout for how rapidly to poll the modem during TX
# if no IRQ is enabled.
#
# All in milliseconds
_RX_POLL_WITH_IRQ = const(1000)
_RX_POLL_NO_IRQ = const(50)
_TX_POLL_LATE_IRQ = const(10)


class AsyncModem:
    # Mixin-like base class that provides coroutines for modem send, recv & recv_continuous.
    #
    # Don't instantiate this class directly, instantiate one of the 'AsyncXYZ'
    # modem classes defined in the lora module.

    def _after_init(self):
        # Separate flags for (rx, tx) as only one task can block on a flag
        self._flags = (asyncio.ThreadSafeFlag(), asyncio.ThreadSafeFlag())
        self.set_irq_callback(self._callback)

    async def recv(self, timeout_ms=None, rx_length=0xFF, rx_packet=None):
        # Async function to receive a single LoRa packet, with an optional timeout
        #
        # Same API as the "Simple API" synchronous function
        self._flags[0].clear()
        will_irq = self.start_recv(timeout_ms, False, rx_length)
        return await self._recv(will_irq, rx_packet)

    def recv_continuous(self, rx_length=0xFF, rx_packet=None):
        # Returns an Async Iterator that continuously yields LoRa packets, until
        # the modem is told to sleep() or standby().
        #
        # Once MicroPython has PEP525 support (PR #6668) then this somewhat
        # fiddly implementation can probably be merged with recv() into a much simpler
        # single _recv() private coro that either yields or returns, depending on
        # an argument. The public API can stay the same.
        will_irq = self.start_recv(None, True, rx_length)
        return AsyncContinuousReceiver(self, will_irq, rx_packet)

    async def _recv(self, will_irq, rx_packet):
        # Second half of receiving is implemented here to share code with AsyncContinuousReceiver,
        # until future refactor is possible (see comment immediately above this one.)
        rx = True
        while rx is True:
            await self._wait(will_irq, 0, _RX_POLL_WITH_IRQ if will_irq else _RX_POLL_NO_IRQ)
            rx = self.poll_recv(rx_packet)
        if rx:  # RxPacket instance
            return rx

    async def send(self, packet, tx_at_ms=None):
        self._flags[1].clear()
        self.prepare_send(packet)

        timeout_ms = self.get_time_on_air_us(len(packet)) // 1000 + 50

        if tx_at_ms is not None:
            await asyncio.sleep_ms(max(0, time.ticks_diff(tx_at_ms, time.ticks_ms())))

        if _DEBUG:
            print("start_send")

        will_irq = self.start_send()

        tx = True
        while tx is True:
            await self._wait(will_irq, 1, timeout_ms)
            tx = self.poll_send()

            if _DEBUG:
                print(f"poll_send returned tx={tx}")

            # If we've already waited the estimated send time (plus 50ms) and the modem
            # is not done, timeout and poll more rapidly from here on (unsure if
            # this is necessary, outside of a serious bug, but doesn't hurt.)
            timeout_ms = _TX_POLL_LATE_IRQ

        return tx

    async def _wait(self, will_irq, idx, timeout_ms):
        # Common code path for blocking on an interrupt, if configured.
        #
        # idx is into _flags tuple and is 0 for rx and 1 for tx
        #
        # timeout_ms is the expected send time for sends, or a reasonable
        # polling timeout for receives.
        if _DEBUG:
            print(f"wait will_irq={will_irq} timeout_ms={timeout_ms}")
        if will_irq:
            # In theory we don't need to ever timeout on the flag as the ISR will always
            # fire, but this gives a workaround for possible race conditions or dropped interrupts.
            try:
                await asyncio.wait_for_ms(self._flags[idx].wait(), timeout_ms)
            except asyncio.TimeoutError:
                pass
        else:
            await asyncio.sleep_ms(timeout_ms)

        if _DEBUG:
            print("wait complete")

    def _callback(self):
        # IRQ callback from BaseModem._radio_isr. May be in Hard IRQ context.
        #
        # Set both RX & TX flag. This isn't necessary for "real" interrupts, but may be necessary
        # to wake both for the case of a "soft" interrupt triggered by sleep() or standby(), where
        # both tasks need to unblock and return.
        #
        # The poll_recv() method is a no-op if _tx is set, so waking up both
        # blocked tasks doesn't have much overhead (no second polling of modem
        # IRQ flags, for example).
        for f in self._flags:
            f.set()


class AsyncContinuousReceiver:
    # Stop-gap async iterator implementation until PEP525 support comes in, see
    # longer comment  on AsyncModem.recv_continuous() above.
    def __init__(self, modem, will_irq, rx_packet):
        self.modem = modem
        self.will_irq = will_irq
        self.rx_packet = rx_packet

    def __aiter__(self):
        return self

    async def __anext__(self):
        res = await self.modem._recv(self.will_irq, self.rx_packet)
        if not res:
            raise StopAsyncIteration
        return res



import errno
import os
import random
import shutil

_ascii_letters = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"


def _get_candidate_name(size=8):
    return "".join(random.choice(_ascii_letters) for _ in range(size))


def _sanitize_inputs(suffix, prefix, dir):
    if dir is None:
        dir = "/tmp"
    if suffix is None:
        suffix = ""
    if prefix is None:
        prefix = ""
    return suffix, prefix, dir


def _try(action, *args, **kwargs):
    try:
        action(*args, **kwargs)
        return True
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise e
    return False


def mkdtemp(suffix=None, prefix=None, dir=None):
    suffix, prefix, dir = _sanitize_inputs(suffix, prefix, dir)

    _try(os.mkdir, dir)

    while True:
        name = _get_candidate_name()
        file = dir + "/" + prefix + name + suffix
        if _try(os.mkdir, file):
            return file


class TemporaryDirectory:
    def __init__(self, suffix=None, prefix=None, dir=None):
        self.name = mkdtemp(suffix, prefix, dir)

    def __repr__(self):
        return "<{} {!r}>".format(self.__class__.__name__, self.name)

    def __enter__(self):
        return self.name

    def __exit__(self, exc, value, tb):
        self.cleanup()

    def cleanup(self):
        _try(shutil.rmtree, self.name)



from pkg.mod import foo

try:
    pkg
except NameError:
    print("NameError")
try:
    pkg.mod
except NameError:
    print("NameError")
print(foo())

# Import few times, must be same module objects
mod_1 = __import__("pkg.mod", None, None, ("foo",))
mod_2 = __import__("pkg.mod", None, None, ("foo",))
print(mod_1 is mod_2)
print(mod_1.foo is mod_2.foo)
print(foo is mod_1.foo)



# Test for VfsLittle using a RAM device

try:
    import os, vfs

    vfs.VfsLfs1
    vfs.VfsLfs2
except (ImportError, AttributeError):
    print("SKIP")
    raise SystemExit


class RAMBlockDevice:
    ERASE_BLOCK_SIZE = 1024

    def __init__(self, blocks):
        self.data = bytearray(blocks * self.ERASE_BLOCK_SIZE)

    def readblocks(self, block, buf, off):
        addr = block * self.ERASE_BLOCK_SIZE + off
        for i in range(len(buf)):
            buf[i] = self.data[addr + i]

    def writeblocks(self, block, buf, off):
        addr = block * self.ERASE_BLOCK_SIZE + off
        for i in range(len(buf)):
            self.data[addr + i] = buf[i]

    def ioctl(self, op, arg):
        if op == 4:  # block count
            return len(self.data) // self.ERASE_BLOCK_SIZE
        if op == 5:  # block size
            return self.ERASE_BLOCK_SIZE
        if op == 6:  # erase block
            return 0


def print_stat(st, print_size=True):
    # don't print times (just check that they have the correct type)
    print(st[:6], st[6] if print_size else -1, type(st[7]), type(st[8]), type(st[9]))


def test(bdev, vfs_class):
    print("test", vfs_class)

    # mkfs
    vfs_class.mkfs(bdev)

    # construction
    fs = vfs_class(bdev)

    # statvfs
    print(fs.statvfs("/"))

    # open, write close
    f = fs.open("test", "w")
    f.write("littlefs")
    f.close()

    # statvfs after creating a file
    print(fs.statvfs("/"))

    # ilistdir
    print(list(fs.ilistdir()))
    print(list(fs.ilistdir("/")))
    print(list(fs.ilistdir(b"/")))

    # mkdir, rmdir
    fs.mkdir("testdir")
    print(list(fs.ilistdir()))
    print(sorted(list(fs.ilistdir("testdir"))))
    fs.rmdir("testdir")
    print(list(fs.ilistdir()))
    fs.mkdir("testdir")

    # stat a file
    print_stat(fs.stat("test"))

    # stat a dir (size seems to vary on LFS2 so don't print that)
    print_stat(fs.stat("testdir"), False)

    # read
    with fs.open("test", "r") as f:
        print(f.read())

    # create large file
    with fs.open("testbig", "w") as f:
        data = "large012" * 32 * 16
        print("data length:", len(data))
        for i in range(4):
            print("write", i)
            f.write(data)

    # stat after creating large file
    print(fs.statvfs("/"))

    # rename
    fs.rename("testbig", "testbig2")
    print(sorted(list(fs.ilistdir())))
    fs.chdir("testdir")
    fs.rename("/testbig2", "testbig2")
    print(sorted(list(fs.ilistdir())))
    fs.rename("testbig2", "/testbig2")
    fs.chdir("/")
    print(sorted(list(fs.ilistdir())))

    # remove
    fs.remove("testbig2")
    print(sorted(list(fs.ilistdir())))

    # getcwd, chdir
    fs.mkdir("/testdir2")
    fs.mkdir("/testdir/subdir")
    print(fs.getcwd())
    fs.chdir("/testdir")
    print(fs.getcwd())

    # create file in directory to make sure paths are relative
    fs.open("test2", "w").close()
    print_stat(fs.stat("test2"))
    print_stat(fs.stat("/testdir/test2"))
    fs.remove("test2")

    # chdir back to root and remove testdir
    fs.chdir("/")
    print(fs.getcwd())
    fs.chdir("testdir")
    print(fs.getcwd())
    fs.chdir("..")
    print(fs.getcwd())
    fs.chdir("testdir/subdir")
    print(fs.getcwd())
    fs.chdir("../..")
    print(fs.getcwd())
    fs.chdir("/./testdir2")
    print(fs.getcwd())
    fs.chdir("../testdir")
    print(fs.getcwd())
    fs.chdir("../..")
    print(fs.getcwd())
    fs.chdir(".//testdir")
    print(fs.getcwd())
    fs.chdir("subdir/./")
    print(fs.getcwd())
    fs.chdir("/")
    print(fs.getcwd())
    fs.rmdir("testdir/subdir")
    fs.rmdir("testdir")
    fs.rmdir("testdir2")


bdev = RAMBlockDevice(30)
test(bdev, vfs.VfsLfs1)
test(bdev, vfs.VfsLfs2)



# flash LED #1 using inline assembler
# this version is overly verbose and uses word stores
#
# ruff: noqa: F821 - @asm_thumb decorator adds names to function scope


@micropython.asm_thumb
def flash_led(r0):
    movw(r1, (stm.GPIOA + stm.GPIO_BSRRL) & 0xFFFF)
    movt(r1, ((stm.GPIOA + stm.GPIO_BSRRL) >> 16) & 0x7FFF)
    movw(r2, 1 << 13)
    movt(r2, 0)
    movw(r3, 0)
    movt(r3, 1 << 13)

    b(loop_entry)

    label(loop1)

    # turn LED on
    str(r2, [r1, 0])

    # delay for a bit
    movw(r4, 5599900 & 0xFFFF)
    movt(r4, (5599900 >> 16) & 0xFFFF)
    label(delay_on)
    sub(r4, r4, 1)
    cmp(r4, 0)
    bgt(delay_on)

    # turn LED off
    str(r3, [r1, 0])

    # delay for a bit
    movw(r4, 5599900 & 0xFFFF)
    movt(r4, (5599900 >> 16) & 0xFFFF)
    label(delay_off)
    sub(r4, r4, 1)
    cmp(r4, 0)
    bgt(delay_off)

    # loop r0 times
    sub(r0, r0, 1)
    label(loop_entry)
    cmp(r0, 0)
    bgt(loop1)


# flash LED #2 using inline assembler
# this version uses half-word sortes, and the convenience assembler operation 'movwt'
@micropython.asm_thumb
def flash_led_v2(r0):
    # get the GPIOA address in r1
    movwt(r1, stm.GPIOA)

    # get the bit mask for PA14 (the pin LED #2 is on)
    movw(r2, 1 << 14)

    b(loop_entry)

    label(loop1)

    # turn LED on
    strh(r2, [r1, stm.GPIO_BSRRL])

    # delay for a bit
    movwt(r4, 5599900)
    label(delay_on)
    sub(r4, r4, 1)
    cmp(r4, 0)
    bgt(delay_on)

    # turn LED off
    strh(r2, [r1, stm.GPIO_BSRRH])

    # delay for a bit
    movwt(r4, 5599900)
    label(delay_off)
    sub(r4, r4, 1)
    cmp(r4, 0)
    bgt(delay_off)

    # loop r0 times
    sub(r0, r0, 1)
    label(loop_entry)
    cmp(r0, 0)
    bgt(loop1)


flash_led(5)
flash_led_v2(5)



# Check that micropython.RingIO works correctly.

import micropython

try:
    micropython.RingIO
except AttributeError:
    print("SKIP")
    raise SystemExit

rb = micropython.RingIO(16)
print(rb)

print(rb.any())

rb.write(b"\x00")
print(rb.any())

rb.write(b"\x00")
print(rb.any())

print(rb.read(2))
print(rb.any())


rb.write(b"\x00\x01")
print(rb.read())

print(rb.read(1))

# Try to write more data than can fit at one go.
print(rb.write(b"\x00\x01" * 10))
print(rb.write(b"\x00"))
print(rb.read())


ba = bytearray(17)
rb = micropython.RingIO(ba)
print(rb)
print(rb.write(b"\x00\x01" * 10))
print(rb.write(b"\x00"))
print(rb.read())

try:
    # Size must be int.
    micropython.RingIO(None)
except TypeError as ex:
    print(type(ex))



"""
NEARC - NEAR Python Contract Compiler

This package provides tools to compile Python smart contracts to WebAssembly
for deployment on the NEAR blockchain.
"""

from .cli import main

__version__ = "0.3.3"

__all__ = ["main"]



# Check optional byteorder argument (CPython 3.11+)
print((10).to_bytes(1))
print((100).to_bytes(10))
print(int.from_bytes(b"\0\0\0\0\0\0\0\0\0\x01"))
print(int.from_bytes(b"\x01\0"))

# Check optional length argument (CPython 3.11+)
print((10).to_bytes())
print((100).to_bytes())



# MicroPython aioble module
# MIT license; Copyright (c) 2021 Jim Mussared

from micropython import const, schedule
import asyncio
import binascii
import json

from .core import log_info, log_warn, ble, register_irq_handler
from .device import DeviceConnection

_IRQ_ENCRYPTION_UPDATE = const(28)
_IRQ_GET_SECRET = const(29)
_IRQ_SET_SECRET = const(30)
_IRQ_PASSKEY_ACTION = const(31)

_IO_CAPABILITY_DISPLAY_ONLY = const(0)
_IO_CAPABILITY_DISPLAY_YESNO = const(1)
_IO_CAPABILITY_KEYBOARD_ONLY = const(2)
_IO_CAPABILITY_NO_INPUT_OUTPUT = const(3)
_IO_CAPABILITY_KEYBOARD_DISPLAY = const(4)

_PASSKEY_ACTION_INPUT = const(2)
_PASSKEY_ACTION_DISP = const(3)
_PASSKEY_ACTION_NUMCMP = const(4)

_DEFAULT_PATH = "ble_secrets.json"

_secrets = {}
_modified = False
_path = None


# Must call this before stack startup.
def load_secrets(path=None):
    global _path, _secrets

    # Use path if specified, otherwise use previous path, otherwise use
    # default path.
    _path = path or _path or _DEFAULT_PATH

    # Reset old secrets.
    _secrets = {}
    try:
        with open(_path, "r") as f:
            entries = json.load(f)
            for sec_type, key, value in entries:
                # Decode bytes from hex.
                _secrets[sec_type, binascii.a2b_base64(key)] = binascii.a2b_base64(value)
    except:
        log_warn("No secrets available")


# Call this whenever the secrets dict changes.
def _save_secrets(arg=None):
    global _modified, _path

    _path = _path or _DEFAULT_PATH

    if not _modified:
        # Only save if the secrets changed.
        return

    with open(_path, "w") as f:
        # Convert bytes to hex strings (otherwise JSON will treat them like
        # strings).
        json_secrets = [
            (sec_type, binascii.b2a_base64(key), binascii.b2a_base64(value))
            for (sec_type, key), value in _secrets.items()
        ]
        json.dump(json_secrets, f)
        _modified = False


def _security_irq(event, data):
    global _modified

    if event == _IRQ_ENCRYPTION_UPDATE:
        # Connection has updated (usually due to pairing).
        conn_handle, encrypted, authenticated, bonded, key_size = data
        log_info("encryption update", conn_handle, encrypted, authenticated, bonded, key_size)
        if connection := DeviceConnection._connected.get(conn_handle, None):
            connection.encrypted = encrypted
            connection.authenticated = authenticated
            connection.bonded = bonded
            connection.key_size = key_size
            # TODO: Handle failure.
            if encrypted and connection._pair_event:
                connection._pair_event.set()

    elif event == _IRQ_SET_SECRET:
        sec_type, key, value = data
        key = sec_type, bytes(key)
        value = bytes(value) if value else None

        log_info("set secret:", key, value)

        if value is None:
            # Delete secret.
            if key not in _secrets:
                return False

            del _secrets[key]
        else:
            # Save secret.
            _secrets[key] = value

        # Queue up a save (don't synchronously write to flash).
        _modified = True
        schedule(_save_secrets, None)

        return True

    elif event == _IRQ_GET_SECRET:
        sec_type, index, key = data

        log_info("get secret:", sec_type, index, bytes(key) if key else None)

        if key is None:
            # Return the index'th secret of this type.
            i = 0
            for (t, _key), value in _secrets.items():
                if t == sec_type:
                    if i == index:
                        return value
                    i += 1
            return None
        else:
            # Return the secret for this key (or None).
            key = sec_type, bytes(key)
            return _secrets.get(key, None)

    elif event == _IRQ_PASSKEY_ACTION:
        conn_handle, action, passkey = data
        log_info("passkey action", conn_handle, action, passkey)
        # if action == _PASSKEY_ACTION_NUMCMP:
        #     # TODO: Show this passkey and confirm accept/reject.
        #     accept = 1
        #     self._ble.gap_passkey(conn_handle, action, accept)
        # elif action == _PASSKEY_ACTION_DISP:
        #     # TODO: Generate and display a passkey so the remote device can enter it.
        #     passkey = 123456
        #     self._ble.gap_passkey(conn_handle, action, passkey)
        # elif action == _PASSKEY_ACTION_INPUT:
        #     # TODO: Ask the user to enter the passkey shown on the remote device.
        #     passkey = 123456
        #     self._ble.gap_passkey(conn_handle, action, passkey)
        # else:
        #     log_warn("unknown passkey action")


def _security_shutdown():
    global _secrets, _modified, _path
    _secrets = {}
    _modified = False
    _path = None


register_irq_handler(_security_irq, _security_shutdown)


# Use device.pair() rather than calling this directly.
async def pair(
    connection,
    bond=True,
    le_secure=True,
    mitm=False,
    io=_IO_CAPABILITY_NO_INPUT_OUTPUT,
    timeout_ms=20000,
):
    ble.config(bond=bond, le_secure=le_secure, mitm=mitm, io=io)

    with connection.timeout(timeout_ms):
        connection._pair_event = asyncio.ThreadSafeFlag()
        ble.gap_pair(connection._conn_handle)
        await connection._pair_event.wait()
        # TODO: Allow the passkey action to return to here and
        # invoke a callback or task to process the action.



#
# Example usage where "P3" is the Buzzer pin.
#
#  from musictest import play
#  play("P3")
#

from machine import Pin
import music


def play(pin_str):
    p = Pin(pin_str, mode=Pin.OUT)
    music.play(music.PRELUDE, pin=p)



# Test MTU exchange (initiated by central) and the effect on notify and write
# size.
#
# See ble_mtu_peripheral.py which tests peripheral-initiated MTU exchange (not supported on btstack).
#
# Four connections are made:
#
# Test | Requested | Preferred | Result | Notes
#   0  |  300 (C)  |  256 (P)  |  256   |
#   1  |  300 (C)  |  200 (P)  |  200   |
#   2  |  300 (C)  |  400 (P)  |  300   |
#   3  |  300 (C)  |  50  (P)  |  50    | Shorter than 64 so the notification is truncated.
#
# For each connection a notification is sent by the server (peripheral) and a characteristic
# is written by the client (central) to ensure that the expected size is transmitted.

from micropython import const
import time, machine, bluetooth

TIMEOUT_MS = 5000

_IRQ_CENTRAL_CONNECT = const(1)
_IRQ_CENTRAL_DISCONNECT = const(2)
_IRQ_GATTS_WRITE = const(3)
_IRQ_PERIPHERAL_CONNECT = const(7)
_IRQ_PERIPHERAL_DISCONNECT = const(8)
_IRQ_GATTC_CHARACTERISTIC_RESULT = const(11)
_IRQ_GATTC_CHARACTERISTIC_DONE = const(12)
_IRQ_GATTC_WRITE_DONE = const(17)
_IRQ_GATTC_NOTIFY = const(18)
_IRQ_MTU_EXCHANGED = const(21)

SERVICE_UUID = bluetooth.UUID("A5A5A5A5-FFFF-9999-1111-5A5A5A5A5A5A")
CHAR_UUID = bluetooth.UUID("00000000-1111-2222-3333-444444444444")
CHAR = (
    CHAR_UUID,
    bluetooth.FLAG_READ | bluetooth.FLAG_WRITE | bluetooth.FLAG_NOTIFY,
)
SERVICE = (
    SERVICE_UUID,
    (CHAR,),
)
SERVICES = (SERVICE,)

waiting_events = {}


def irq(event, data):
    if event == _IRQ_CENTRAL_CONNECT:
        print("_IRQ_CENTRAL_CONNECT")
        waiting_events[event] = data[0]
    elif event == _IRQ_CENTRAL_DISCONNECT:
        print("_IRQ_CENTRAL_DISCONNECT")
    elif event == _IRQ_GATTS_WRITE:
        print("_IRQ_GATTS_WRITE")
    elif event == _IRQ_PERIPHERAL_CONNECT:
        print("_IRQ_PERIPHERAL_CONNECT")
        waiting_events[event] = data[0]
    elif event == _IRQ_PERIPHERAL_DISCONNECT:
        print("_IRQ_PERIPHERAL_DISCONNECT")
    elif event == _IRQ_GATTC_CHARACTERISTIC_RESULT:
        if data[-1] == CHAR_UUID:
            print("_IRQ_GATTC_CHARACTERISTIC_RESULT", data[-1])
            waiting_events[event] = data[2]
        else:
            return
    elif event == _IRQ_GATTC_CHARACTERISTIC_DONE:
        print("_IRQ_GATTC_CHARACTERISTIC_DONE")
    elif event == _IRQ_GATTC_WRITE_DONE:
        print("_IRQ_GATTC_WRITE_DONE")
    elif event == _IRQ_GATTC_NOTIFY:
        print("_IRQ_GATTC_NOTIFY", len(data[-1]), chr(data[-1][0]))
    elif event == _IRQ_MTU_EXCHANGED:
        print("_IRQ_MTU_EXCHANGED", data[-1])
        waiting_events[event] = data[-1]

    if event not in waiting_events:
        waiting_events[event] = None


def wait_for_event(event, timeout_ms):
    t0 = time.ticks_ms()
    while time.ticks_diff(time.ticks_ms(), t0) < timeout_ms:
        if event in waiting_events:
            return waiting_events.pop(event)
        machine.idle()
    raise ValueError("Timeout waiting for {}".format(event))


# Acting in peripheral role.
def instance0():
    multitest.globals(BDADDR=ble.config("mac"))
    ((char_handle,),) = ble.gatts_register_services(SERVICES)
    ble.gatts_set_buffer(char_handle, 500, False)
    multitest.next()

    default_mtus = [256, 200, 400, 50]
    try:
        for i in range(4):
            ble.config(mtu=default_mtus[i])

            print("gap_advertise")
            ble.gap_advertise(20_000, b"\x02\x01\x06\x04\xffMPY")
            multitest.broadcast(f"peripheral:adv:{i}")

            # Wait for central to connect to us.
            conn_handle = wait_for_event(_IRQ_CENTRAL_CONNECT, TIMEOUT_MS)

            mtu = wait_for_event(_IRQ_MTU_EXCHANGED, TIMEOUT_MS)

            multitest.wait(f"client:discovery:{i}")

            print("gatts_notify")
            ble.gatts_notify(conn_handle, char_handle, str(i) * 64)

            wait_for_event(_IRQ_GATTS_WRITE, TIMEOUT_MS)

            print("gatts_read")
            data = ble.gatts_read(char_handle)
            print("characteristic len:", len(data), chr(data[0]))

            # Wait for the central to disconnect.
            wait_for_event(_IRQ_CENTRAL_DISCONNECT, TIMEOUT_MS)
    finally:
        ble.active(0)


# Acting in central role.
def instance1():
    multitest.next()
    try:
        for i in range(4):
            ble.config(mtu=300)

            multitest.wait(f"peripheral:adv:{i}")

            print("gap_connect")
            ble.gap_connect(BDADDR[0], BDADDR[1], TIMEOUT_MS)
            conn_handle = wait_for_event(_IRQ_PERIPHERAL_CONNECT, TIMEOUT_MS)

            # Central-initiated mtu exchange.
            print("gattc_exchange_mtu")
            ble.gattc_exchange_mtu(conn_handle)

            mtu = wait_for_event(_IRQ_MTU_EXCHANGED, TIMEOUT_MS)

            print("gattc_discover_characteristics")
            ble.gattc_discover_characteristics(conn_handle, 1, 65535)
            value_handle = wait_for_event(_IRQ_GATTC_CHARACTERISTIC_RESULT, TIMEOUT_MS)
            wait_for_event(_IRQ_GATTC_CHARACTERISTIC_DONE, TIMEOUT_MS)

            multitest.broadcast(f"client:discovery:{i}")

            wait_for_event(_IRQ_GATTC_NOTIFY, TIMEOUT_MS)

            print("gattc_write")
            ble.gattc_write(conn_handle, value_handle, chr(ord("a") + i) * (mtu - 3), 1)
            wait_for_event(_IRQ_GATTC_WRITE_DONE, TIMEOUT_MS)

            time.sleep_ms(300)

            # Disconnect from peripheral.
            print("gap_disconnect:", ble.gap_disconnect(conn_handle))
            wait_for_event(_IRQ_PERIPHERAL_DISCONNECT, TIMEOUT_MS)
    finally:
        ble.active(0)


ble = bluetooth.BLE()
ble.active(1)
ble.irq(irq)



import time
from hwconfig import LED


# Using sleep_ms() gives pretty poor PWM resolution and
# brightness control, but we use it in the attempt to
# make this demo portable to even more boards (e.g. to
# those which don't provide sleep_us(), or provide, but
# it's not precise, like would be on non realtime OSes).
# We otherwise use 20ms period, to make frequency not less
# than 50Hz to avoid visible flickering (you may still see
# if you're unlucky).
def pwm_cycle(led, duty, cycles):
    duty_off = 20 - duty
    for i in range(cycles):
        if duty:
            led.on()
            time.sleep_ms(duty)
        if duty_off:
            led.off()
            time.sleep_ms(duty_off)


# At the duty setting of 1, an LED is still pretty bright, then
# at duty 0, it's off. This makes rather unsmooth transition, and
# breaks fade effect. So, we avoid value of 0 and oscillate between
# 1 and 20. Actually, highest values like 19 and 20 are also
# barely distinguishible (like, both of them too bright and burn
# your eye). So, improvement to the visible effect would be to use
# more steps (at least 10x), and then higher frequency, and use
# range which includes 1 but excludes values at the top.
while True:
    # Fade in
    for i in range(1, 21):
        pwm_cycle(LED, i, 2)
    # Fade out
    for i in range(20, 0, -1):
        pwm_cycle(LED, i, 2)



# MicroPython USB host library, compatible with PyUSB.
# MIT license; Copyright (c) 2021-2024 Damien P. George


def get_descriptor(dev, desc_size, desc_type, desc_index, wIndex=0):
    wValue = desc_index | desc_type << 8
    d = dev.ctrl_transfer(0x80, 0x06, wValue, wIndex, desc_size)
    if len(d) < 2:
        raise Exception("invalid descriptor")
    return d



# test that iterating doesn't use the heap
try:
    frozenset
except NameError:
    print("SKIP")
    raise SystemExit
try:
    import array
except ImportError:
    print("SKIP")
    raise SystemExit

try:
    from micropython import heap_lock, heap_unlock
except (ImportError, AttributeError):
    heap_lock = heap_unlock = lambda: 0


def do_iter(l):
    heap_lock()
    for i in l:
        print(i)
    heap_unlock()


def gen_func():
    yield 1
    yield 2


# pre-create collections to iterate over
ba = bytearray(b"123")
ar = array.array("H", (123, 456))
t = (1, 2, 3)
l = [1, 2]
d = {1: 2}
s = set((1,))
fs = frozenset((1,))
g1 = (100 + x for x in range(2))
g2 = gen_func()

# test containment (both success and failure) with the heap locked
heap_lock()
print(49 in b"123", 255 in b"123")
print(1 in t, -1 in t)
print(1 in l, -1 in l)
print(1 in d, -1 in d)
print(1 in s, -1 in s)
heap_unlock()

# test unpacking with the heap locked
unp0 = unp1 = unp2 = None  # preallocate slots for globals
heap_lock()
unp0, unp1, unp2 = t
print(unp0, unp1, unp2)
heap_unlock()

# test certain builtins with the heap locked
heap_lock()
print(all(t))
print(any(t))
print(min(t))
print(max(t))
print(sum(t))
heap_unlock()

# test iterating over collections with the heap locked
do_iter(b"123")
do_iter(ba)
do_iter(ar)
do_iter(t)
do_iter(l)
do_iter(d)
do_iter(s)
do_iter(fs)
do_iter(g1)
do_iter(g2)



# test concurrent mutating access to a shared bytearray object
#
# MIT license; Copyright (c) 2016 Damien P. George on behalf of Pycom Ltd

import _thread

# the shared bytearray
ba = bytearray()


# main thread function
def th(n, lo, hi):
    for repeat in range(n):
        for i in range(lo, hi):
            l = len(ba)
            ba.append(i)
            assert len(ba) >= l + 1

            l = len(ba)
            ba.extend(bytearray([i]))
            assert len(ba) >= l + 1

    with lock:
        global n_finished
        n_finished += 1


lock = _thread.allocate_lock()
n_thread = 4
n_finished = 0
n_repeat = 4  # use 40 for more stressful test (uses more heap)

# spawn threads
for i in range(n_thread):
    _thread.start_new_thread(th, (n_repeat, i * 256 // n_thread, (i + 1) * 256 // n_thread))

# busy wait for threads to finish
while n_finished < n_thread:
    pass

# check bytearray has correct contents
print(len(ba))
count = [0 for _ in range(256)]
for b in ba:
    count[b] += 1
print(count)



"""
categories: Core,Classes
description: When inheriting native types, calling a method in ``__init__(self, ...)`` before ``super().__init__()`` raises an ``AttributeError`` (or segfaults if ``MICROPY_BUILTIN_METHOD_CHECK_SELF_ARG`` is not enabled).
cause: MicroPython does not have separate ``__new__`` and ``__init__`` methods in native types.
workaround: Call ``super().__init__()`` first.
"""


class L1(list):
    def __init__(self, a):
        self.append(a)


try:
    L1(1)
    print("OK")
except AttributeError:
    print("AttributeError")


class L2(list):
    def __init__(self, a):
        super().__init__()
        self.append(a)


try:
    L2(1)
    print("OK")
except AttributeError:
    print("AttributeError")



# Test TCP server with client issuing TCP RST part way through read

try:
    import asyncio
except ImportError:
    print("SKIP")
    raise SystemExit

import struct, time, socket

PORT = 8000


async def handle_connection(reader, writer):
    data = await reader.read(10)  # should succeed
    print(data)
    await asyncio.sleep(0.2)  # wait for client to drop connection
    try:
        data = await reader.read(100)
        print(data)
        writer.close()
        await writer.wait_closed()
    except OSError as er:
        print("OSError", er.errno)
    ev.set()


async def main():
    global ev
    ev = asyncio.Event()
    server = await asyncio.start_server(handle_connection, "0.0.0.0", PORT)
    multitest.next()
    async with server:
        await asyncio.wait_for(ev.wait(), 10)


def instance0():
    multitest.globals(IP=multitest.get_network_ip())
    asyncio.run(main())


def instance1():
    if not hasattr(socket, "SO_LINGER"):
        multitest.skip()
    multitest.next()
    s = socket.socket()
    s.connect(socket.getaddrinfo(IP, PORT)[0][-1])
    lgr_onoff = 1
    lgr_linger = 0
    s.setsockopt(socket.SOL_SOCKET, socket.SO_LINGER, struct.pack("ii", lgr_onoff, lgr_linger))
    s.send(b"GET / HTTP/1.0\r\n\r\n")
    time.sleep(0.1)
    s.close()  # This issues a TCP RST since we've set the linger option



# basic list functionality
x = [1, 2, 3 * 4]
print(x)
x[0] = 4
print(x)
x[1] += -4
print(x)
x.append(5)
print(x)
f = x.append
f(4)
print(x)

x.extend([100, 200])
print(x)
x.extend(range(3))
print(x)

x += [2, 1]
print(x)

# unsupported type on RHS of add
try:
    [] + None
except TypeError:
    print('TypeError')



# Extension for "unittest" that adds the ability to run via "micropython -m unittest".

import argparse
import os
import sys
from fnmatch import fnmatch
from micropython import const

try:
    from unittest import TestRunner, TestResult, TestSuite
except ImportError:
    print("Error: This must be used from an installed copy of unittest-discover which will"
          " also install base unittest module.")
    raise


# Run a single test in a clean environment.
def _run_test_module(runner: TestRunner, module_name: str, *extra_paths: list[str]):
    module_snapshot = {k: v for k, v in sys.modules.items()}
    path_snapshot = sys.path[:]
    try:
        for path in extra_paths:
            if path:
                sys.path.insert(0, path)

        module = __import__(module_name, None, None, module_name)
        suite = TestSuite(module_name)
        suite._load_module(module)
        return runner.run(suite)
    finally:
        sys.path[:] = path_snapshot
        sys.modules.clear()
        sys.modules.update(module_snapshot)


_DIR_TYPE = const(0x4000)


def _run_all_in_dir(runner: TestRunner, path: str, pattern: str, top: str):
    result = TestResult()
    for fname, ftype, *_ in os.ilistdir(path):
        if fname in ("..", "."):
            continue
        fpath = "/".join((path, fname))
        if ftype == _DIR_TYPE:
            result += _run_all_in_dir(
                runner=runner,
                path=fpath,
                pattern=pattern,
                top=top,
            )
        if fnmatch(fname, pattern):
            module_path = fpath.rsplit(".", 1)[0]  # remove ext
            module_path = module_path.replace("/", ".").strip(".")
            result += _run_test_module(runner, module_path, top)
    return result


# Implements discovery inspired by https://docs.python.org/3/library/unittest.html#test-discovery
def _discover(runner: TestRunner):
    parser = argparse.ArgumentParser()
    # parser.add_argument(
    #     "-v",
    #     "--verbose",
    #     action="store_true",
    #     help="Verbose output",
    # )
    parser.add_argument(
        "-s",
        "--start-directory",
        dest="start",
        default=".",
        help="Directory to start discovery",
    )
    parser.add_argument(
        "-p",
        "--pattern ",
        dest="pattern",
        default="test*.py",
        help="Pattern to match test files",
    )
    parser.add_argument(
        "-t",
        "--top-level-directory",
        dest="top",
        help="Top level directory of project (defaults to start directory)",
    )
    args = parser.parse_args(args=sys.argv[2:])

    path = args.start
    top = args.top or path

    return _run_all_in_dir(
        runner=runner,
        path=path,
        pattern=args.pattern,
        top=top,
    )


# TODO: Use os.path for path handling.
PATH_SEP = getattr(os, "sep", "/")


# foo/bar/x.y.z --> foo/bar, x.y
def _dirname_filename_no_ext(path):
    # Workaround: The Windows port currently reports "/" for os.sep
    # (and MicroPython doesn't have os.altsep). So for now just
    # always work with os.sep (i.e. "/").
    path = path.replace("\\", PATH_SEP)

    split = path.rsplit(PATH_SEP, 1)
    if len(split) == 1:
        dirname, filename = "", split[0]
    else:
        dirname, filename = split
    return dirname, filename.rsplit(".", 1)[0]


def discover_main():
    runner = TestRunner()

    if len(sys.argv) == 1 or (
        len(sys.argv) >= 2
        and _dirname_filename_no_ext(sys.argv[0])[1] == "unittest"
        and sys.argv[1] == "discover"
    ):
        # No args, or `python -m unittest discover ...`.
        result = _discover(runner)
    else:
        result = TestResult()
        for test_spec in sys.argv[1:]:
            try:
                os.stat(test_spec)
                # File exists, strip extension and import with its parent directory in sys.path.
                dirname, module_name = _dirname_filename_no_ext(test_spec)
                res = _run_test_module(runner, module_name, dirname)
            except OSError:
                # Not a file, treat as named module to import.
                res = _run_test_module(runner, test_spec)

            result += res

    if not result.testsRun:
        # If tests are run their results are already printed.
        # Ensure an appropriate output is printed if no tests are found.
        runner.run(TestSuite())

    # Terminate with non zero return code in case of failures.
    sys.exit(result.failuresNum + result.errorsNum)


discover_main()



# Miscellaneous viper tests.

import micropython


# a for loop
@micropython.viper
def viper_for(a: int, b: int) -> int:
    total = 0
    for x in range(a, b):
        total += x
    return total


print(viper_for(10, 10000))


# accessing a global
@micropython.viper
def viper_access_global():
    global gl
    gl = 1
    return gl


print(viper_access_global(), gl)


# calling print with object and int types
@micropython.viper
def viper_print(x, y: int):
    print(x, y + 1)


viper_print(1, 2)


# convert constants to objects in tuple
@micropython.viper
def viper_tuple_consts(x):
    return (x, 1, False, True)


print(viper_tuple_consts(0))


# making a tuple from an object and an int
@micropython.viper
def viper_tuple(x, y: int):
    return (x, y + 1)


print(viper_tuple(1, 2))


# making a list from an object and an int
@micropython.viper
def viper_list(x, y: int):
    return [x, y + 1]


print(viper_list(1, 2))


# making a set from an object and an int
@micropython.viper
def viper_set(x, y: int):
    return {x, y + 1}


print(sorted(list(viper_set(1, 2))))


# raising an exception
@micropython.viper
def viper_raise(x: int):
    raise OSError(x)


try:
    viper_raise(1)
except OSError as e:
    print(repr(e))


# calling GC after defining the function
@micropython.viper
def viper_gc() -> int:
    return 1


print(viper_gc())
import gc

gc.collect()
print(viper_gc())



# test builtin hash function

print(hash(False))
print(hash(True))
print({():1}) # hash tuple
print({(1,):1}) # hash non-empty tuple
print(hash in {hash:1}) # hash function
print(type(hash(list.pop))) # hash checked function (mp_type_checked_fun)
print(type(hash([].pop))) # hash bound method
print(type(hash(object()))) # hash object instance
print(type(hash(super(object, object)))) # hash super
print(type(hash(classmethod(hash)))) # hash classmethod
print(type(hash(staticmethod(hash)))) # hash staticmethod
print(type(hash(iter("")))) # hash string iterator
print(type(hash(iter(b"")))) # hash bytes iterator
print(type(hash(iter(range(0))))) # hash range iterator
print(type(hash(map(None, [])))) # hash map object
print(type(hash(zip([])))) # hash zip object

def f(x):
    def g():
        return x
    return g

print(type(hash(f(1)))) # hash closure

try:
    hash([])
except TypeError:
    print("TypeError")

class A:
    def __hash__(self):
        return 123
    def __repr__(self):
        return "a instance"

print(hash(A()))
print({A():1})

# all user-classes have default __hash__
class B:
    pass
hash(B())

# if __eq__ is defined then default __hash__ is not used
class C:
    def __eq__(self, another):
        return True
try:
    hash(C())
except TypeError:
    print("TypeError")

# __hash__ must return an int
class D:
    def __hash__(self):
        return None
try:
    hash(D())
except TypeError:
    print("TypeError")

# __hash__ returning a bool should be converted to an int
class E:
    def __hash__(self):
        return True
print(hash(E()))



# Test performance of importing an .mpy file just once.
# The first import of a module will intern strings that don't already exist, and
# this test should be representative of what happens in a real application.

import sys, io, vfs

if not hasattr(io, "IOBase"):
    print("SKIP")
    raise SystemExit

# This is the test.py file that is compiled to test.mpy below.
# Many known and unknown names/strings are included to test the linking process.
"""
class A0:
    def a0(self): pass
    def a1(self): pass
    def a2(self): pass
    def a3(self): pass
class A1:
    def a0(self): pass
    def a1(self): pass
    def a2(self): pass
    def a3(self): pass
def f0():
    __call__, __class__, __delitem__, __enter__, __exit__, __getattr__, __getitem__,
    __hash__, __init__, __int__, __iter__, __len__, __main__, __module__, __name__,
    __new__, __next__, __qualname__, __repr__, __setitem__, __str__,
    ArithmeticError, AssertionError, AttributeError, BaseException, EOFError, Ellipsis,
    Exception, GeneratorExit, ImportError, IndentationError, IndexError, KeyError,
    KeyboardInterrupt, LookupError, MemoryError, NameError, NoneType,
    NotImplementedError, OSError, OverflowError, RuntimeError, StopIteration,
    SyntaxError, SystemExit, TypeError, ValueError, ZeroDivisionError,
    abs, all, any, append, args, bool, builtins, bytearray, bytecode, bytes, callable,
    chr, classmethod, clear, close, const, copy, count, dict, dir, divmod, end,
    endswith, eval, exec, extend, find, format, from_bytes, get, getattr, globals,
    hasattr, hash, id, index, insert, int, isalpha, isdigit, isinstance, islower,
    isspace, issubclass, isupper, items, iter, join, key, keys, len, list, little,
    locals, lower, lstrip, main, map, micropython, next, object, open, ord, pop,
    popitem, pow, print, range, read, readinto, readline, remove, replace, repr,
    reverse, rfind, rindex, round, rsplit, rstrip, self, send, sep, set, setattr,
    setdefault, sort, sorted, split, start, startswith, staticmethod, step, stop, str,
    strip, sum, super, throw, to_bytes, tuple, type, update, upper, value, values,
    write, zip,
    name0, name1, name2, name3, name4, name5, name6, name7, name8, name9,
    quite_a_long_name0, quite_a_long_name1, quite_a_long_name2, quite_a_long_name3,
    quite_a_long_name4, quite_a_long_name5, quite_a_long_name6, quite_a_long_name7,
    quite_a_long_name8, quite_a_long_name9, quite_a_long_name10, quite_a_long_name11,
def f1():
    x = "this will be a string object 0"
    x = "this will be a string object 1"
    x = "this will be a string object 2"
    x = "this will be a string object 3"
    x = "this will be a string object 4"
    x = "this will be a string object 5"
    x = "this will be a string object 6"
    x = "this will be a string object 7"
    x = "this will be a string object 8"
    x = "this will be a string object 9"
    x = b"this will be a bytes object 0"
    x = b"this will be a bytes object 1"
    x = b"this will be a bytes object 2"
    x = b"this will be a bytes object 3"
    x = b"this will be a bytes object 4"
    x = b"this will be a bytes object 5"
    x = b"this will be a bytes object 6"
    x = b"this will be a bytes object 7"
    x = b"this will be a bytes object 8"
    x = b"this will be a bytes object 9"
    x = ("const tuple 0", None, False, True, 1, 2, 3)
    x = ("const tuple 1", None, False, True, 1, 2, 3)
    x = ("const tuple 2", None, False, True, 1, 2, 3)
    x = ("const tuple 3", None, False, True, 1, 2, 3)
    x = ("const tuple 4", None, False, True, 1, 2, 3)
    x = ("const tuple 5", None, False, True, 1, 2, 3)
    x = ("const tuple 6", None, False, True, 1, 2, 3)
    x = ("const tuple 7", None, False, True, 1, 2, 3)
    x = ("const tuple 8", None, False, True, 1, 2, 3)
    x = ("const tuple 9", None, False, True, 1, 2, 3)
result = 123
"""
file_data = b"M\x06\x00\x1f\x81=\x1e\x0etest.py\x00\x0f\x04A0\x00\x04A1\x00\x04f0\x00\x04f1\x00\x0cresult\x00/-5\x04a0\x00\x04a1\x00\x04a2\x00\x04a3\x00\x13\x15\x17\x19\x1b\x1d\x1f!#%')+1379;=?ACEGIKMOQSUWY[]_acegikmoqsuwy{}\x7f\x81\x01\x81\x03\x81\x05\x81\x07\x81\t\x81\x0b\x81\r\x81\x0f\x81\x11\x81\x13\x81\x15\x81\x17\x81\x19\x81\x1b\x81\x1d\x81\x1f\x81!\x81#\x81%\x81'\x81)\x81+\x81-\x81/\x811\x813\x815\x817\x819\x81;\x81=\x81?\x81A\x81C\x81E\x81G\x81I\x81K\x81M\x81O\x81Q\x81S\x81U\x81W\x81Y\x81[\x81]\x81_\x81a\x81c\x81e\x81g\x81i\x81k\x81m\x81o\x81q\x81s\x81u\x81w\x81y\x81{\x81}\x81\x7f\x82\x01\x82\x03\x82\x05\x82\x07\x82\t\x82\x0b\x82\r\x82\x0f\x82\x11\x82\x13\x82\x15\x82\x17\x82\x19\x82\x1b\x82\x1d\x82\x1f\x82!\x82#\x82%\x82'\x82)\x82+\x82-\x82/\x821\x823\x825\x827\x829\x82;\x82=\x82?\x82A\x82E\x82G\x82I\x82K\nname0\x00\nname1\x00\nname2\x00\nname3\x00\nname4\x00\nname5\x00\nname6\x00\nname7\x00\nname8\x00\nname9\x00$quite_a_long_name0\x00$quite_a_long_name1\x00$quite_a_long_name2\x00$quite_a_long_name3\x00$quite_a_long_name4\x00$quite_a_long_name5\x00$quite_a_long_name6\x00$quite_a_long_name7\x00$quite_a_long_name8\x00$quite_a_long_name9\x00&quite_a_long_name10\x00&quite_a_long_name11\x00\x05\x1ethis will be a string object 0\x00\x05\x1ethis will be a string object 1\x00\x05\x1ethis will be a string object 2\x00\x05\x1ethis will be a string object 3\x00\x05\x1ethis will be a string object 4\x00\x05\x1ethis will be a string object 5\x00\x05\x1ethis will be a string object 6\x00\x05\x1ethis will be a string object 7\x00\x05\x1ethis will be a string object 8\x00\x05\x1ethis will be a string object 9\x00\x06\x1dthis will be a bytes object 0\x00\x06\x1dthis will be a bytes object 1\x00\x06\x1dthis will be a bytes object 2\x00\x06\x1dthis will be a bytes object 3\x00\x06\x1dthis will be a bytes object 4\x00\x06\x1dthis will be a bytes object 5\x00\x06\x1dthis will be a bytes object 6\x00\x06\x1dthis will be a bytes object 7\x00\x06\x1dthis will be a bytes object 8\x00\x06\x1dthis will be a bytes object 9\x00\n\x07\x05\rconst tuple 0\x00\x01\x02\x03\x07\x011\x07\x012\x07\x013\n\x07\x05\rconst tuple 1\x00\x01\x02\x03\x07\x011\x07\x012\x07\x013\n\x07\x05\rconst tuple 2\x00\x01\x02\x03\x07\x011\x07\x012\x07\x013\n\x07\x05\rconst tuple 3\x00\x01\x02\x03\x07\x011\x07\x012\x07\x013\n\x07\x05\rconst tuple 4\x00\x01\x02\x03\x07\x011\x07\x012\x07\x013\n\x07\x05\rconst tuple 5\x00\x01\x02\x03\x07\x011\x07\x012\x07\x013\n\x07\x05\rconst tuple 6\x00\x01\x02\x03\x07\x011\x07\x012\x07\x013\n\x07\x05\rconst tuple 7\x00\x01\x02\x03\x07\x011\x07\x012\x07\x013\n\x07\x05\rconst tuple 8\x00\x01\x02\x03\x07\x011\x07\x012\x07\x013\n\x07\x05\rconst tuple 9\x00\x01\x02\x03\x07\x011\x07\x012\x07\x013\x82d\x10\x12\x01i@i@\x84\x18\x84\x1fT2\x00\x10\x024\x02\x16\x02T2\x01\x10\x034\x02\x16\x032\x02\x16\x042\x03\x16\x05\"\x80{\x16\x06Qc\x04\x82\x0c\x00\n\x02($$$\x11\x07\x16\x08\x10\x02\x16\t2\x00\x16\n2\x01\x16\x0b2\x02\x16\x0c2\x03\x16\rQc\x04@\t\x08\n\x81\x0b Qc@\t\x08\x0b\x81\x0b@Qc@\t\x08\x0c\x81\x0b`QcH\t\n\r\x81\x0b` Qc\x82\x14\x00\x0c\x03h`$$$\x11\x07\x16\x08\x10\x03\x16\t2\x00\x16\n2\x01\x16\x0b2\x02\x16\x0c2\x03\x16\rQc\x04H\t\n\n\x81\x0b``QcH\t\n\x0b\x81\x0b\x80\x07QcH\t\n\x0c\x81\x0b\x80\x08QcH\t\n\r\x81\x0b\x80\tQc\xa08P:\x04\x80\x0b13///---997799<\x1f%\x1f\"\x1f%)\x1f\"//\x12\x0e\x12\x0f\x12\x10\x12\x11\x12\x12\x12\x13\x12\x14*\x07Y\x12\x15\x12\x16\x12\x17\x12\x18\x12\x19\x12\x1a\x12\x08\x12\x07*\x08Y\x12\x1b\x12\x1c\x12\t\x12\x1d\x12\x1e\x12\x1f*\x06Y\x12 \x12!\x12\"\x12#\x12$\x12%*\x06Y\x12&\x12'\x12(\x12)\x12*\x12+*\x06Y\x12,\x12-\x12.\x12/\x120*\x05Y\x121\x122\x123\x124\x125*\x05Y\x126\x127\x128\x129\x12:*\x05Y\x12;\x12<\x12=\x12>\x12?\x12@\x12A\x12B\x12C\x12D\x12E*\x0bY\x12F\x12G\x12H\x12I\x12J\x12K\x12L\x12M\x12N\x12O\x12P*\x0bY\x12Q\x12R\x12S\x12T\x12U\x12V\x12W\x12X\x12Y\x12Z*\nY\x12[\x12\\\x12]\x12^\x12_\x12`\x12a\x12b\x12c\x12d*\nY\x12e\x12f\x12g\x12h\x12i\x12j\x12k\x12l\x12m\x12n\x12o*\x0bY\x12p\x12q\x12r\x12s\x12t\x12u\x12v\x12w\x12x\x12y\x12z*\x0bY\x12{\x12|\x12}\x12~\x12\x7f\x12\x81\x00\x12\x81\x01\x12\x81\x02\x12\x81\x03\x12\x81\x04*\nY\x12\x81\x05\x12\x81\x06\x12\x81\x07\x12\x81\x08\x12\x81\t\x12\x81\n\x12\x81\x0b\x12\x81\x0c\x12\x81\r\x12\x81\x0e\x12\x81\x0f*\x0bY\x12\x81\x10\x12\x81\x11\x12\x81\x12\x12\x81\x13\x12\x81\x14\x12\x81\x15\x12\x81\x16\x12\x81\x17\x12\x81\x18\x12\x81\x19*\nY\x12\x81\x1a\x12\x81\x1b\x12\x81\x1c\x12\x81\x1d\x12\x81\x1e\x12\x81\x1f\x12\x81 \x12\x81!\x12\x81\"\x12\x81#\x12\x81$*\x0bY\x12\x81%\x12\x81&*\x02Y\x12\x81'\x12\x81(\x12\x81)\x12\x81*\x12\x81+\x12\x81,\x12\x81-\x12\x81.\x12\x81/\x12\x810*\nY\x12\x811\x12\x812\x12\x813\x12\x814*\x04Y\x12\x815\x12\x816\x12\x817\x12\x818*\x04Y\x12\x819\x12\x81:\x12\x81;\x12\x81<*\x04YQc\x87p\x08@\x05\x80###############################\x00\xc0#\x01\xc0#\x02\xc0#\x03\xc0#\x04\xc0#\x05\xc0#\x06\xc0#\x07\xc0#\x08\xc0#\t\xc0#\n\xc0#\x0b\xc0#\x0c\xc0#\r\xc0#\x0e\xc0#\x0f\xc0#\x10\xc0#\x11\xc0#\x12\xc0#\x13\xc0#\x14\xc0#\x15\xc0#\x16\xc0#\x17\xc0#\x18\xc0#\x19\xc0#\x1a\xc0#\x1b\xc0#\x1c\xc0#\x1d\xc0Qc"


class File(io.IOBase):
    def __init__(self):
        self.off = 0

    def ioctl(self, request, arg):
        return 0

    def readinto(self, buf):
        buf[:] = memoryview(file_data)[self.off : self.off + len(buf)]
        self.off += len(buf)
        return len(buf)


class FS:
    def mount(self, readonly, mkfs):
        pass

    def chdir(self, path):
        pass

    def stat(self, path):
        if path == "/__injected.mpy":
            return tuple(0 for _ in range(10))
        else:
            raise OSError(-2)  # ENOENT

    def open(self, path, mode):
        return File()


def mount():
    vfs.mount(FS(), "/__remote")
    sys.path.insert(0, "/__remote")


def test():
    global result
    module = __import__("__injected")
    result = module.result


###########################################################################
# Benchmark interface

bm_params = {
    (1, 1): (),
}


def bm_setup(params):
    mount()
    return lambda: test(), lambda: (1, result)



# Miscellaneous viper tests.

import micropython


# viper function taking and returning ints
@micropython.viper
def viper_int(x: int, y: int) -> int:
    return x + y + 3


print(viper_int(1, 2))


# viper function taking and returning objects
@micropython.viper
def viper_object(x: object, y: object) -> object:
    return x + y


print(viper_object(1, 2))


# return None as non-object (should return 0)
@micropython.viper
def viper_ret_none() -> int:
    return None


print(viper_ret_none())


# return Ellipsis as object
@micropython.viper
def viper_ret_ellipsis() -> object:
    return ...


print(viper_ret_ellipsis())


# 3 args
@micropython.viper
def viper_3args(a: int, b: int, c: int) -> int:
    return a + b + c


print(viper_3args(1, 2, 3))


# 4 args
@micropython.viper
def viper_4args(a: int, b: int, c: int, d: int) -> int:
    return a + b + c + d


print(viper_4args(1, 2, 3, 4))


# a local (should have automatic type int)
@micropython.viper
def viper_local(x: int) -> int:
    y = 4
    return x + y


print(viper_local(3))


# without type annotation, types should default to object
@micropython.viper
def viper_no_annotation(x, y):
    return x * y


print(viper_no_annotation(4, 5))



import sqlite3


def test_autocommit():
    conn = sqlite3.connect(":memory:", autocommit=True)

    # First cursor creates table and inserts value (DML)
    cur = conn.cursor()
    cur.execute("CREATE TABLE foo(a int)")
    cur.execute("INSERT INTO foo VALUES (42)")
    cur.close()

    # Second cursor fetches 42 due to the autocommit
    cur = conn.cursor()
    cur.execute("SELECT * FROM foo")
    assert cur.fetchone() == (42,)
    assert cur.fetchone() is None

    cur.close()
    conn.close()

def test_manual():
    conn = sqlite3.connect(":memory:", autocommit=False)

    # First cursor creates table, insert rolls back
    cur = conn.cursor()
    cur.execute("CREATE TABLE foo(a int)")
    conn.commit()
    cur.execute("INSERT INTO foo VALUES (42)")
    cur.close()
    conn.rollback()

    # Second connection fetches nothing due to the rollback
    cur = conn.cursor()
    cur.execute("SELECT * FROM foo")
    assert cur.fetchone() is None

    cur.close()
    conn.close()

test_autocommit()
test_manual()
